"      NAME     Isard
       AUTHOR   jean-sebastien.roy@wanadoo.fr
       URL      
       FUNCTION Interactive Geometry Package
       ST-VERSIONS      VisualWorks 2.5, VisualWorks 3.0
       PREREQUISITES    (none)
       CONFLICTS        (none known)
       DISTRIBUTION     world
       VERSION  f1.0
       DATE     06-Nov-99

SUMMARY

Isard is geometry package I developed after learning Smalltalk in 1996-1997. 
It provides an interface (like Cabri-Geometre) to create geometric figures and is fully dynamic. 


                                Jean-Sebastien Roy
"!

Dictionary variableSubclass: #IsardDictionnaireMethodesDeCreation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardDictionnaireMethodesDeCreation comment:
'Dictionnaire de methodes de creation'!


!IsardDictionnaireMethodesDeCreation methodsFor: 'adding'!

ajouteMethodeDeCreation: mdc 
	"ajoute la mthode mdc au dictionnaire"
	"le dico a le nombre d'args de chaque mthode comme cl"
	"et une ordered collection de mdc comme value"

	(self at: mdc arguments size ifAbsentPut: [OrderedCollection new])
		add: mdc! !

!IsardDictionnaireMethodesDeCreation methodsFor: 'menu'!

menuDeCreationPourLaSelection: selection ajouteAuMenu: menuBuilder 
	"ajoute a menuBuilder un menu liste des mdc compatibles avec la selection"

	| mdcc dico |
	mdcc := self methodesDeCreationCompatiblesAvec: selection.
	mdcc isEmpty ifTrue: [^0].
	dico := Dictionary new.
	mdcc do: [:mdc | (dico at: mdc classeCree nomType ifAbsentPut: [OrderedCollection new])
			add: mdc].
	dico
		keysAndValuesDo: 
			[:type :mdct | 
			menuBuilder beginSubMenuLabeled: type.
			mdct do: [:mdc | menuBuilder add: (mdc classeCree nomSpecifiqueAvecParents: selection)
						-> [:r | r choisitCree: (mdc realiseAvecArguments: selection)]].
			menuBuilder endSubMenu].
	^mdcc size! !

!IsardDictionnaireMethodesDeCreation methodsFor: 'compatibilite'!

methodesDeCreationCompatiblesAvec: selection 
	"determine l'ensemble des mdc compatibles avec une selection"
	"(arguments passs dans l'ordre)"

	| o c |
	o := OrderedCollection new.
	c := self at: selection size ifAbsent: [^o].
	c do: [:mdc | (mdc arguments estCompatibleAvec: selection)
			ifTrue: [o add: mdc]].
	^o! !

Object subclass: #IsardMethodeDeCreation
	instanceVariableNames: 'selecteur arguments classeCree '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMethodeDeCreation comment:
'une methode de creation

Instance Variables:
	arguments	<IsardArgumentsMethodeDeCreation>	arguments de la mdc
	classeCree	<Metaclass>	classe cree par la mdc
	selecteur	<ByteSymbol>	selecteur de la mdc

'!


!IsardMethodeDeCreation methodsFor: 'accessing'!

arguments
	^arguments!

arguments: anObject
	arguments := anObject!

classeCree
	^classeCree!

classeCree: anObject
	classeCree := anObject!

selecteur
	^selecteur!

selecteur: anObject
	selecteur := anObject! !

!IsardMethodeDeCreation methodsFor: 'realisation'!

realiseAvecArguments: args 
	"ralise la mdc avec args comme arguments"

	^self classeCree perform: self selecteur withArguments: args asArray! !

!IsardMethodeDeCreation methodsFor: 'printing'!

messageDeCreationAvecParametresNommes: npar 
	"affiche:"
	"classeCree selecteur1: parametre1 selecteur2: parametre2 [...]."
	"en fait, classeCree est remplacee par l'implementeur reel du selecteur"

	| str |
	str := WriteStream on: String new.
	str nextPutAll: (classeCree class whichClassIncludesSelector: selecteur) soleInstance name.
	selecteur keywords with: npar do: [:k :n | str space; nextPutAll: k; space; nextPutAll: n].
	str nextPutAll: '.'.
	^str contents!

printOn: aStream 
	aStream nextPutAll: (self messageDeCreationAvecParametresNommes: (arguments collect: [:a | a name]))! !

View subclass: #IsardView
	instanceVariableNames: 'echelle coin limites buffer aJour buffered '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardView comment:
'vue d''un IsardPlan

Instance Variables:
	aJour	<Boolean>	vrai si le plan n''a subit aucune modif depuis le dernier invalidate de la vue
	buffer	<Object>	pixmap contenant une copie de la vue pour le tripleBuffering
	buffered	<Boolean>	vrai si le buffer contient effectivement une copie a jour
	limites	<rect>	rect de la vue en coordonnees du modele
	coin	<Object>	coord dans le plan du coin superieur gauche de la vue
	echelle	<Object>	facteur multiplicatif des distances entre le plan et la vue

'!


!IsardView methodsFor: 'updating'!

update
	self update: #trace with: nil!

update: anAspect with: anObject 
	anAspect = #trace ifTrue: [aJour := false].
	self invalidate! !

!IsardView methodsFor: 'displaying'!

displayOn: gc 
	aJour 
		ifTrue: 
			[(buffered and: [ buffer isOpen and: [buffer extent = self bounds extent]]) 
				ifFalse: 
					[(buffer notNil and: [ buffer isOpen and: [buffer extent = self bounds extent]]) 
						ifFalse: [buffer := Pixmap extent: self bounds extent].
					buffer clear.
					self miseAJourCoinEtEchelle.
					model aRetracerPour: self sur: buffer graphicsContext.
					buffered := true].
			buffer displayOn: gc]
		ifFalse: 
			[gc clear.
			self miseAJourCoinEtEchelle.
			model aRetracerPour: self sur: gc.
			aJour := true.
			buffered := false]!

imprimerDansLeFichier: filename 
	"imprime taille max, centre, au format landscape sur une page A4"
	"en theorie...!! (ie: chez moi ;-)"

	| ps gc s sc |
	ps := PostScriptFile named: filename.
	ps doEPS: true.
	ps pageSize: 21 @ 29.7 / 2.54.
	ps setLandscape: true.
	gc := ps graphicsContext.
	s := gc clippingBounds extent / self bounds extent.
	sc := s x max: s y.
	gc translateBy: gc clippingBounds extent - (self bounds extent * sc) / 2.
	gc scaleBy: sc.
	(filename , ' (Isard, (c) Jean-Sbastien ROY, 1997-1999)') asComposedText displayOn: gc at: 20 @ 20.
	model aRetracerPour: self sur: gc.
	ps close! !

!IsardView methodsFor: 'private'!

calculeLimites
	limites := coin corner: (self invTrans: self bounds corner)!

coin
	^coin!

coin: c 
	limites := c extent: limites extent.
	self update!

echelle
	^echelle!

echelle: e 
	echelle := e.
	self calculeLimites.
	self update!

miseAJourCoinEtEchelle
	| echellep |
	echellep := 1 @ -1 * (self bounds extent / limites extent).
	echelle := echellep x min: echellep y.
	coin := (limites origin + limites corner) / 2 
				- (self bounds extent / 2 * (1 @ -1) / echelle).
	self calculeLimites! !

!IsardView methodsFor: 'point de vue'!

centre: p 
	self limites: (p - (limites extent / 2) extent: limites extent)!

faitCorrespondre: pmodel avec: pvue 
	"fait correspondre le point pmodel dans les coord du modele"
	"avec pvue dans les coord de la vue"

	self coin: pmodel - (pvue * (1 @ -1) / echelle)!

limites
	^limites!

limites: rect 
	limites := rect.
	self update!

zoom: facteur 
	self 
		limites: (limites origin + (limites extent * (1 - (1 / facteur)) / 2) 
				extent: limites extent / facteur)! !

!IsardView methodsFor: 'mise a l''echelle'!

distTrans: d 
	"passe des coordonnees du modele vers les coordonnees de la vue"
	"d est ici une distance"

	^d * echelle!

invDistTrans: d 
	"passe des coordonnees de la vue vers les coordonnees du modele"
	"d est ici une distance"

	^d / echelle!

invTrans: p 
	"passe des coordonnees la vue vers les coordonnees du modele"

	^p * (1 @ -1) / echelle + coin!

trans: p 
	"passe des coordonnees du modele vers les coordonnees de la vue"

	^p - coin * (1 @ -1) * echelle! !

!IsardView methodsFor: 'controller accessing'!

defaultControllerClass
	^IsardController! !

!IsardView methodsFor: 'initialization'!

initialize
	aJour := false.
	buffered := false.
	super initialize! !

ApplicationModel subclass: #IsardPlanDisplay2
	instanceVariableNames: 'view1 view2 model limitesInitiales1 limitesInitiales2 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!


!IsardPlanDisplay2 methodsFor: 'aspects'!

isardView1
	^view1!

isardView2
	^view2! !

!IsardPlanDisplay2 methodsFor: 'accessing'!

model
	^model!

model: anObject
	model := anObject! !

!IsardPlanDisplay2 methodsFor: 'opening'!

plan: p limites: l 
	view1 := IsardView new.
	view1 model: p.
	view1 limites: l.
	view2 := IsardView new.
	view2 model: p.
	view2 limites: l.
	self open!

postBuildWith: b 
	"on change de damageRepairPolicy pour ne pas avoir de clignotements"

	super postBuildWith: b.
	b window damageRepairPolicy: DoubleBufferingWindowDisplayPolicy new! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPlanDisplay2 class
	instanceVariableNames: ''!


!IsardPlanDisplay2 class methodsFor: 'opening'!

plan: p 
	^self plan: p limites: (-15 @ 15 corner: 15 @ -15)!

plan: p limites: l 
	^self new plan: p limites: l!

vierge
	^self plan: IsardPlan new! !

!IsardPlanDisplay2 class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Visualiseur d''Isard-Plan' 
			#min: #(#Point 300 300 ) 
			#bounds: #(#Rectangle 480 265 1119 744 ) 
			#colors: 
			#(#LookPreferences 
				#setBackgroundColor: #(#ColorValue #white ) 
				#setBorderColor: #(#ColorValue #black ) ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ArbitraryComponentSpec 
					#layout: #(#LayoutFrame 5 0 5 0 -5 0.5 -5 1 ) 
					#component: #isardView1 ) 
				#(#ArbitraryComponentSpec 
					#layout: #(#LayoutFrame 5 0.5 5 0 -5 1 -5 1 ) 
					#component: #isardView2 ) ) ) )! !

ApplicationModel subclass: #IsardPlanDisplay
	instanceVariableNames: 'view model limitesInitiales '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPlanDisplay comment:
'Application affichant la vue IsardView

Instance Variables:
	model	<IsardPlan>	modele (un plan)
	view	<IsardView>	vue

'!


!IsardPlanDisplay methodsFor: 'aspects'!

isardView
	^view! !

!IsardPlanDisplay methodsFor: 'opening'!

plan: p limites: l 
	view := IsardView new.
	view model: p.
	view limites: l.
	self open!

postBuildWith: b 
	"on change de damageRepairPolicy pour ne pas avoir de clignotements"

	super postBuildWith: b.
	b window damageRepairPolicy: DoubleBufferingWindowDisplayPolicy new! !

!IsardPlanDisplay methodsFor: 'accessing'!

model
	^model!

model: anObject
	model := anObject! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPlanDisplay class
	instanceVariableNames: ''!


!IsardPlanDisplay class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Visualiseur d''Isard-Plan' 
			#min: #(#Point 300 300 ) 
			#bounds: #(#Rectangle 321 241 960 720 ) 
			#colors: 
			#(#LookPreferences 
				#setBackgroundColor: #(#ColorValue #white ) 
				#setBorderColor: #(#ColorValue #black ) ) 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ArbitraryComponentSpec 
					#layout: #(#LayoutFrame 5 0 5 0 -5 1 -5 1 ) 
					#component: #isardView ) ) ) )! !

!IsardPlanDisplay class methodsFor: 'opening'!

plan: p 
	^self plan: p limites: (-15 @ 15 corner: 15 @ -15)!

plan: p limites: l 
	^self new plan: p limites: l!

vierge
	^self plan: IsardPlan new! !

Object subclass: #IsardTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardTests comment:
'contient l''ensemble des methodes sauvees par l''application plan display dans sa classe'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardTests class
	instanceVariableNames: ''!


!IsardTests class methodsFor: 'tests'!

cercleEulerOrig
	"IsardTests cercleEulerOrig"
	| plan cpta cptb cptc pta ptb ptc dab ha dac dbc hb hc sab sbc ap bp sac cp a1 b1 c1 h a2 b2 c2 sha shb shc mab mbc mac o oh op c9 cc |
	plan := IsardPlan new.
	cpta := IsardControleCoord p: 1 @ 7.5 plan: plan.
	pta := (IsardPoint coord: cpta)
				nom: 'A'.
	cptb := IsardControleCoord p: -9 @ -4 plan: plan.
	ptb := (IsardPoint coord: cptb)
				nom: 'B'.
	cptc := IsardControleCoord p: 7 @ -4 plan: plan.
	ptc := (IsardPoint coord: cptc)
				nom: 'C'.
	dab := IsardDroite passantPar: pta etPar: ptb.
	dac := IsardDroite passantPar: pta etPar: ptc.
	dbc := IsardDroite passantPar: ptb etPar: ptc.
	sab := IsardSegment joignant: pta avec: ptb.
	sbc := IsardSegment joignant: ptb avec: ptc.
	sac := IsardSegment joignant: ptc avec: pta.
	ha := IsardDroite perpendiculaireA: dbc passantPar: pta.
	hb := IsardDroite perpendiculaireA: dac passantPar: ptb.
	hc := IsardDroite perpendiculaireA: dab passantPar: ptc.
	ha couleur: ColorValue blue.
	hb couleur: ColorValue blue.
	hc couleur: ColorValue blue.
	ap := IsardPoint milieuDe: sbc.
	bp := IsardPoint milieuDe: sac.
	cp := IsardPoint milieuDe: sab.
	ap nom: 'A'''.
	bp nom: 'B'''.
	cp nom: 'C'''.
	a1 := IsardPoint intersectionDeLaDroite: ha avecLaDroite: dbc.
	b1 := IsardPoint intersectionDeLaDroite: hb avecLaDroite: dac.
	c1 := IsardPoint intersectionDeLaDroite: hc avecLaDroite: dab.
	a1 nom: 'A1'.
	b1 nom: 'B1'.
	c1 nom: 'C1'.
	h := IsardPoint intersectionDeLaDroite: ha avecLaDroite: hb.
	h nom: 'H'.
	sha := IsardSegment joignant: h avec: pta.
	shb := IsardSegment joignant: h avec: ptb.
	shc := IsardSegment joignant: h avec: ptc.
	sha invisible.
	shb invisible.
	shc invisible.
	a2 := IsardPoint milieuDe: sha.
	b2 := IsardPoint milieuDe: shb.
	c2 := IsardPoint milieuDe: shc.
	a2 nom: 'A2'.
	b2 nom: 'B2'.
	c2 nom: 'C2'.
	mab := IsardDroite mediatriceDe: sab.
	mbc := IsardDroite mediatriceDe: sbc.
	mac := IsardDroite mediatriceDe: sac.
	mab couleur: ColorValue red.
	mbc couleur: ColorValue red.
	mac couleur: ColorValue red.
	o := IsardPoint intersectionDeLaDroite: mab avecLaDroite: mbc.
	o nom: 'O'.
	cc := IsardCercle deCentre: o passantPar: pta.
	cc couleur: ColorValue royalBlue.
	oh := IsardSegment joignant: o avec: h.
	op := IsardPoint milieuDe: oh.
	op nom: 'O'''.
	c9 := IsardCercle deCentre: op passantPar: a1.
	c9 couleur: ColorValue darkGreen.
	c9 epaisseur: 2.
	IsardPlanDisplay plan: plan!

cercleEulerOrig2
	| plan cpta cptb cptc pta ptb ptc dab ha dac dbc hb hc sab sbc ap bp sac cp a1 b1 c1 h a2 b2 c2 sha shb shc mab mbc mac o oh op c9 cc |
	plan := IsardPlan new.
	cpta := IsardControleCoord p: 1 @ 7.5 plan: plan.
	pta := (IsardPoint coord: cpta)
				nom: 'A'.
	cptb := IsardControleCoord p: -9 @ -4 plan: plan.
	ptb := (IsardPoint coord: cptb)
				nom: 'B'.
	cptc := IsardControleCoord p: 7 @ -4 plan: plan.
	ptc := (IsardPoint coord: cptc)
				nom: 'C'.
	dab := IsardDroite passantPar: pta etPar: ptb.
	dac := IsardDroite passantPar: pta etPar: ptc.
	dbc := IsardDroite passantPar: ptb etPar: ptc.
	sab := IsardSegment joignant: pta avec: ptb.
	sbc := IsardSegment joignant: ptb avec: ptc.
	sac := IsardSegment joignant: ptc avec: pta.
	ha := IsardDroite perpendiculaireA: dbc passantPar: pta.
	hb := IsardDroite perpendiculaireA: dac passantPar: ptb.
	hc := IsardDroite perpendiculaireA: dab passantPar: ptc.
	ha couleur: ColorValue blue.
	hb couleur: ColorValue blue.
	hc couleur: ColorValue blue.
	ap := IsardPoint milieuDe: sbc.
	bp := IsardPoint milieuDe: sac.
	cp := IsardPoint milieuDe: sab.
	ap nom: 'A'''.
	bp nom: 'B'''.
	cp nom: 'C'''.
	a1 := IsardPoint intersectionDeLaDroite: ha avecLaDroite: dbc.
	b1 := IsardPoint intersectionDeLaDroite: hb avecLaDroite: dac.
	c1 := IsardPoint intersectionDeLaDroite: hc avecLaDroite: dab.
	a1 nom: 'A1'.
	b1 nom: 'B1'.
	c1 nom: 'C1'.
	h := IsardPoint intersectionDeLaDroite: ha avecLaDroite: hb.
	h nom: 'H'.
	sha := IsardSegment joignant: h avec: pta.
	shb := IsardSegment joignant: h avec: ptb.
	shc := IsardSegment joignant: h avec: ptc.
	sha invisible.
	shb invisible.
	shc invisible.
	a2 := IsardPoint milieuDe: sha.
	b2 := IsardPoint milieuDe: shb.
	c2 := IsardPoint milieuDe: shc.
	a2 nom: 'A2'.
	b2 nom: 'B2'.
	c2 nom: 'C2'.
	mab := IsardDroite mediatriceDe: sab.
	mbc := IsardDroite mediatriceDe: sbc.
	mac := IsardDroite mediatriceDe: sac.
	mab couleur: ColorValue red.
	mbc couleur: ColorValue red.
	mac couleur: ColorValue red.
	o := IsardPoint intersectionDeLaDroite: mab avecLaDroite: mbc.
	o nom: 'O'.
	cc := IsardCercle deCentre: o passantPar: pta.
	cc couleur: ColorValue royalBlue.
	oh := IsardSegment joignant: o avec: h.
	op := IsardPoint milieuDe: oh.
	op nom: 'O'''.
	c9 := IsardCercle deCentre: op passantPar: a1.
	c9 couleur: ColorValue darkGreen.
	c9 epaisseur: 2.
	IsardPlanDisplay2 plan: plan!

limaconOrig
	| plan cpto o r c posA a doa cptu u dab dub b l posC cptr |
	plan := IsardPlan new.
	cpto := IsardControleCoord p: 0 @ 0 plan: plan.
	o := IsardPoint coord: cpto.
	o nom: 'O'.
	cptr := IsardControleCoord p: 0 @ 2 plan: plan.
	r := IsardPoint coord: cptr.
	r nom: 'R'.
	c := IsardCercle deCentre: o passantPar: r.
	posA := IsardControleNombre n: 0.3 plan: plan.
	a := IsardPoint surLeCercle: c controlePar: posA.
	a nom: 'A'.
	doa := IsardDroite passantPar: o etPar: a.
	cptu := IsardControleCoord p: -3 @ 0 plan: plan.
	u := IsardPoint coord: cptu.
	u nom: 'U'.
	dab := IsardDroite perpendiculaireA: doa passantPar: a.
	dub := IsardDroite perpendiculaireA: dab passantPar: u.
	b := IsardPoint intersectionDeLaDroite: dab avecLaDroite: dub.
	b nom: 'B'.
	l := IsardLieu duPoint: b controlePar: posA.
	l nom: 'Limacon de pascal'.
	l couleur: ColorValue red.
	posC := IsardControleNombre n: 0.3 plan: plan.
	c := IsardPoint surLeLieu: l controlePar: posC.
	c nom: 'C'.
	IsardPlanDisplay plan: plan!

testOrig
	| plan cpta cptb cptc pta ptb ptc sab sbc sca mab mbc ccc cc |
	plan := IsardPlan new.
	cpta := IsardControleCoord p: -1 @ -2 plan: plan.
	pta := (IsardPoint coord: cpta)
				nom: 'A'.
	cptb := IsardControleCoord p: 1 @ 0 plan: plan.
	ptb := (IsardPoint coord: cptb)
				nom: 'B'.
	cptc := IsardControleCoord p: 0.5 @ 1 plan: plan.
	ptc := (IsardPoint coord: cptc)
				nom: 'C'.
	sab := IsardSegment joignant: pta avec: ptb.
	sbc := IsardSegment joignant: ptb avec: ptc.
	sca := IsardSegment joignant: ptc avec: pta.
	mab := (IsardDroite mediatriceDe: sab)
				couleur: ColorValue red.
	mbc := (IsardDroite mediatriceDe: sbc)
				couleur: ColorValue red.
	ccc := IsardPoint intersectionDeLaDroite: mab avecLaDroite: mbc.
	cc := IsardCercle deCentre: ccc passantPar: pta.
	cc couleur: ColorValue blue.
	cc epaisseur: 2.
	IsardPlanDisplay plan: plan!

tousTests
	IsardPlanDisplay plan: IsardTests cercleEulerSauve limites: (-86 / 5 @ 11 corner: 76 / 5 @ -13).
	IsardPlanDisplay plan: IsardTests limaconSauve limites: (-89 / 10 @ (59 / 10) corner: 73 / 10 @ (-61 / 10)).
	IsardPlanDisplay plan: IsardTests menelausSauve limites: (-89 / 10 @ (59 / 10) corner: 73 / 10 @ (-61 / 10)).
	IsardPlanDisplay plan: IsardTests paraboleSauve limites: (-81 / 4 @ 15 corner: 81 / 4 @ -15).
	IsardPlanDisplay plan: IsardTests problemeManuSauve limites: (-9.85 @ 5.75 corner: 6.35 @ -6.25).
	IsardPlanDisplay plan: IsardTests triMed limites: (-81 / 4 @ 15 corner: 81 / 4 @ -15)! !

!IsardTests class methodsFor: 'IsardResources'!

bissectrices
	"Executer la commande suivante pour editer l'objet:
	IsardPlanDisplay
		plan: IsardTests bissectrices
		limites: ((-177/8)@(133/8) corner: (147/8)@(-107/8))
	"

	| plan Coordonne1 Point1 Coordonne2 Point2 Coordonne3 Point3 Droite1 Droite2 Droite3 Point4 Segment1 Segment2 Segment3 Droite4 Droite5 Point5 Cercle1 Transformation1 Point6 Droite6 Point7 Droite7 Point8 Cercle2 Droite8 Droite9 Transformation2 Point9 Droite10 Point10 Droite11 Point11 Cercle3 Point12 Droite12 Point13 Cercle4 | 
	plan :=  IsardPlan new. plan nomVar: 'plan'.
	Coordonne1 := IsardControleCoord p: (-55/16)@(27/4) plan: plan.
	Point1 := IsardPoint coord: Coordonne1.
	Point1 nom: 'A'.
	Coordonne2 := IsardControleCoord p: (1/16)@(9/16) plan: plan.
	Point2 := IsardPoint coord: Coordonne2.
	Point2 nom: 'B'.
	Coordonne3 := IsardControleCoord p: (-97/16)@(5/4) plan: plan.
	Point3 := IsardPoint coord: Coordonne3.
	Point3 nom: 'C'.
	Droite1 := IsardDroite bissectriceDe: Point1 et: Point2 et: Point3.
	Droite2 := IsardDroite bissectriceDe: Point2 et: Point1 et: Point3.
	Droite3 := IsardDroite bissectriceDe: Point1 et: Point3 et: Point2.
	Point4 := IsardPoint intersectionDeLaDroite: Droite2 avecLaDroite: Droite1.
	Point4 nom: 'J'.
	Segment1 := IsardSegment joignant: Point1 avec: Point2.
	Segment1 couleur: ColorValue red ; epaisseur: 2.
	Segment2 := IsardSegment joignant: Point1 avec: Point3.
	Segment2 couleur: ColorValue red ; epaisseur: 2.
	Segment3 := IsardSegment joignant: Point3 avec: Point2.
	Segment3 couleur: ColorValue red ; epaisseur: 2.
	Droite4 := IsardDroite passantPar: Point2 etPar: Point3.
	Droite4 couleur: ColorValue red ; epaisseur: 2.
	Droite5 := IsardDroite perpendiculaireA: Droite4 passantPar: Point4.
	Droite5 visibilite: false.
	Point5 := IsardPoint intersectionDeLaDroite: Droite4 avecLaDroite: Droite5.
	Point5 visibilite: false ; nom: 'D'.
	Cercle1 := IsardCercle deCentre: Point4 passantPar: Point5.
	Cercle1 couleur: ColorValue royalBlue ; epaisseur: 2.
	Transformation1 := IsardSymetrieCentrale deCentre: Point2.
	Point6 := IsardPoint transformationDe: Point1 par: Transformation1.
	Point6 visibilite: false.
	Droite6 := IsardDroite bissectriceDe: Point3 et: Point2 et: Point6.
	Droite6 visibilite: false.
	Point7 := IsardPoint intersectionDeLaDroite: Droite2 avecLaDroite: Droite6.
	Point7 nom: 'JA'.
	Droite7 := IsardDroite perpendiculaireA: Droite4 passantPar: Point7.
	Droite7 visibilite: false.
	Point8 := IsardPoint intersectionDeLaDroite: Droite7 avecLaDroite: Droite4.
	Point8 visibilite: false ; nom: 'DA'.
	Cercle2 := IsardCercle deCentre: Point7 passantPar: Point8.
	Cercle2 couleur: ColorValue blue ; epaisseur: 2.
	Droite8 := IsardDroite passantPar: Point1 etPar: Point3.
	Droite8 couleur: ColorValue red ; epaisseur: 2.
	Droite9 := IsardDroite passantPar: Point1 etPar: Point2.
	Droite9 couleur: ColorValue red ; epaisseur: 2.
	Transformation2 := IsardSymetrieCentrale deCentre: Point3.
	Point9 := IsardPoint transformationDe: Point2 par: Transformation2.
	Point9 visibilite: false.
	Droite10 := IsardDroite bissectriceDe: Point9 et: Point3 et: Point1.
	Droite10 visibilite: false.
	Point10 := IsardPoint intersectionDeLaDroite: Droite10 avecLaDroite: Droite1.
	Point10 nom: 'JB'.
	Droite11 := IsardDroite perpendiculaireA: Droite4 passantPar: Point10.
	Droite11 visibilite: false.
	Point11 := IsardPoint intersectionDeLaDroite: Droite11 avecLaDroite: Droite4.
	Point11 visibilite: false.
	Cercle3 := IsardCercle deCentre: Point10 passantPar: Point11.
	Cercle3 couleur: ColorValue blue ; epaisseur: 2.
	Point12 := IsardPoint intersectionDeLaDroite: Droite3 avecLaDroite: Droite6.
	Point12 nom: 'JC'.
	Droite12 := IsardDroite perpendiculaireA: Droite8 passantPar: Point12.
	Droite12 visibilite: false.
	Point13 := IsardPoint intersectionDeLaDroite: Droite8 avecLaDroite: Droite12.
	Point13 visibilite: false.
	Cercle4 := IsardCercle deCentre: Point12 passantPar: Point13.
	Cercle4 couleur: ColorValue blue ; epaisseur: 2.
	^plan!

cercleEulerSauve
	"Executer la commande suivante pour editer l'objet:
	IsardPlanDisplay
		plan: IsardTests cercleEulerSauve
		limites: ((-86/5)@11 corner: (76/5)@-13)
	"

	| plan Controle1 Point1 Controle2 Point2 Controle3 Point3 Droite1 Droite2 Droite3 Segment1 Segment2 Segment3 Droite4 Droite5 Droite6 Point4 Point5 Point6 Point7 Point8 Point9 Point10 Segment4 Segment5 Segment6 Point11 Point12 Point13 Droite7 Droite8 Droite9 Point14 Cercle1 Segment7 Point15 Cercle2 | 
	plan :=  IsardPlan new. plan nomVar: 'plan'.
	Controle1 := IsardControleCoord p: (5/4)@(97/10) plan: plan.
	Point1 := IsardPoint coord: Controle1.
	Point1 nom: 'A'.
	Controle2 := IsardControleCoord p: (-109/10)@(-63/10) plan: plan.
	Point2 := IsardPoint coord: Controle2.
	Point2 nom: 'B'.
	Controle3 := IsardControleCoord p: (68/5)@(-47/5) plan: plan.
	Point3 := IsardPoint coord: Controle3.
	Point3 nom: 'C'.
	Droite1 := IsardDroite passantPar: Point1 etPar: Point2.
	Droite2 := IsardDroite passantPar: Point1 etPar: Point3.
	Droite3 := IsardDroite passantPar: Point2 etPar: Point3.
	Segment1 := IsardSegment joignant: Point1 avec: Point2.
	Segment2 := IsardSegment joignant: Point2 avec: Point3.
	Segment3 := IsardSegment joignant: Point3 avec: Point1.
	Droite4 := IsardDroite perpendiculaireA: Droite3 passantPar: Point1.
	Droite4 couleur: ColorValue blue.
	Droite5 := IsardDroite perpendiculaireA: Droite2 passantPar: Point2.
	Droite5 couleur: ColorValue blue.
	Droite6 := IsardDroite perpendiculaireA: Droite1 passantPar: Point3.
	Droite6 couleur: ColorValue blue.
	Point4 := IsardPoint milieuDe: Segment2.
	Point4 nom: 'A'''.
	Point5 := IsardPoint milieuDe: Segment3.
	Point5 nom: 'B'''.
	Point6 := IsardPoint milieuDe: Segment1.
	Point6 nom: 'C'''.
	Point7 := IsardPoint intersectionDeLaDroite: Droite4 avecLaDroite: Droite3.
	Point7 nom: 'A1'.
	Point8 := IsardPoint intersectionDeLaDroite: Droite5 avecLaDroite: Droite2.
	Point8 nom: 'B1'.
	Point9 := IsardPoint intersectionDeLaDroite: Droite6 avecLaDroite: Droite1.
	Point9 nom: 'C1'.
	Point10 := IsardPoint intersectionDeLaDroite: Droite4 avecLaDroite: Droite5.
	Point10 nom: 'H'.
	Segment4 := IsardSegment joignant: Point10 avec: Point1.
	Segment4 visibilite: false.
	Segment5 := IsardSegment joignant: Point10 avec: Point2.
	Segment5 visibilite: false.
	Segment6 := IsardSegment joignant: Point10 avec: Point3.
	Segment6 visibilite: false.
	Point11 := IsardPoint milieuDe: Segment4.
	Point11 nom: 'A2'.
	Point12 := IsardPoint milieuDe: Segment5.
	Point12 nom: 'B2'.
	Point13 := IsardPoint milieuDe: Segment6.
	Point13 nom: 'C2'.
	Droite7 := IsardDroite mediatriceDe: Segment1.
	Droite7 couleur: ColorValue red.
	Droite8 := IsardDroite mediatriceDe: Segment2.
	Droite8 couleur: ColorValue red.
	Droite9 := IsardDroite mediatriceDe: Segment3.
	Droite9 couleur: ColorValue red.
	Point14 := IsardPoint intersectionDeLaDroite: Droite7 avecLaDroite: Droite8.
	Point14 nom: 'O'.
	Cercle1 := IsardCercle deCentre: Point14 passantPar: Point1.
	Cercle1 couleur: ColorValue royalBlue.
	Segment7 := IsardSegment joignant: Point14 avec: Point10.
	Point15 := IsardPoint milieuDe: Segment7.
	Point15 nom: 'O'''.
	Cercle2 := IsardCercle deCentre: Point15 passantPar: Point7.
	Cercle2 couleur: ColorValue darkGreen ; epaisseur: 2.
	^plan!

limaconSauve
	"Executer la commande suivante pour editer l'objet:
	IsardPlanDisplay
		plan: IsardTests limaconSauve
		limites: ((-89/10)@(59/10) corner: (73/10)@(-61/10))
	"

	| plan coord Point1 Controle2 Point2 Cercle1 Controle3 Point3 Droite1 Controle4 Point4 Droite2 Droite3 Point5 Lieu1 Controle5 Point6 | 
	plan :=  IsardPlan new. plan nomVar: 'plan'.
	coord := IsardControleCoord p: 0@0 plan: plan.
	coord nomVar: 'coord'.
	Point1 := IsardPoint coord: coord.
	Point1 nom: 'O'.
	Controle2 := IsardControleCoord p: 0@2 plan: plan.
	Point2 := IsardPoint coord: Controle2.
	Point2 nom: 'R'.
	Cercle1 := IsardCercle deCentre: Point1 passantPar: Point2.
	Controle3 := IsardControleNombre n: 0.3 plan: plan.
	Point3 := IsardPoint surLeCercle: Cercle1 controlePar: Controle3.
	Point3 nom: 'A'.
	Droite1 := IsardDroite passantPar: Point1 etPar: Point3.
	Controle4 := IsardControleCoord p: -3@0 plan: plan.
	Point4 := IsardPoint coord: Controle4.
	Point4 nom: 'U'.
	Droite2 := IsardDroite perpendiculaireA: Droite1 passantPar: Point3.
	Droite3 := IsardDroite perpendiculaireA: Droite2 passantPar: Point4.
	Point5 := IsardPoint intersectionDeLaDroite: Droite2 avecLaDroite: Droite3.
	Point5 nom: 'B'.
	Lieu1 := IsardLieu duPoint: Point5 controlePar: Controle3.
	Lieu1 couleur: ColorValue red ; nom: 'Limacon de pascal'.
	Controle5 := IsardControleNombre n: 0.3 plan: plan.
	Point6 := IsardPoint surLeLieu: Lieu1 controlePar: Controle5.
	Point6 nom: 'C'.
	^plan!

menelausSauve
	"Executer la commande suivante pour editer l'objet:
	IsardPlanDisplay
		plan: IsardTests menelausSauve
		limites: ((-89/10)@(59/10) corner: (73/10)@(-61/10))
	"

	| plan Controle1 Point1 Controle2 Point2 Controle3 Point3 Droite1 Droite2 Droite3 Controle4 Point4 Controle5 Point5 Droite4 Point6 Point7 Point8 Segment1 Point9 Segment2 Point10 Segment3 Point11 Segment4 Point12 Segment5 Point13 Segment6 Point14 Droite5 Droite6 Droite7 Droite8 | 
	plan :=  IsardPlan new. plan nomVar: 'plan'.
	Controle1 := IsardControleCoord p: (-9/5)@(-143/40) plan: plan.
	Point1 := IsardPoint coord: Controle1.
	Point1 nom: 'B'.
	Controle2 := IsardControleCoord p: (101/40)@(-143/40) plan: plan.
	Point2 := IsardPoint coord: Controle2.
	Point2 nom: 'C'.
	Controle3 := IsardControleCoord p: (-8/5)@(11/8) plan: plan.
	Point3 := IsardPoint coord: Controle3.
	Point3 nom: 'A'.
	Droite1 := IsardDroite passantPar: Point1 etPar: Point2.
	Droite2 := IsardDroite passantPar: Point3 etPar: Point2.
	Droite3 := IsardDroite passantPar: Point3 etPar: Point1.
	Controle4 := IsardControleCoord p: 2@(27/20) plan: plan.
	Point4 := IsardPoint coord: Controle4.
	Point4 nom: 'D2'.
	Controle5 := IsardControleCoord p: (-321/40)@(-93/20) plan: plan.
	Point5 := IsardPoint coord: Controle5.
	Point5 nom: 'D1'.
	Droite4 := IsardDroite passantPar: Point4 etPar: Point5.
	Droite4 couleur: ColorValue blue.
	Point6 := IsardPoint intersectionDeLaDroite: Droite4 avecLaDroite: Droite1.
	Point6 nom: 'D'.
	Point7 := IsardPoint intersectionDeLaDroite: Droite3 avecLaDroite: Droite4.
	Point7 nom: 'F'.
	Point8 := IsardPoint intersectionDeLaDroite: Droite2 avecLaDroite: Droite4.
	Point8 nom: 'E'.
	Segment1 := IsardSegment joignant: Point3 avec: Point6.
	Segment1 couleur: ColorValue red.
	Point9 := IsardPoint milieuDe: Segment1.
	Point9 nom: 'M1'.
	Segment2 := IsardSegment joignant: Point1 avec: Point8.
	Segment2 couleur: ColorValue red.
	Point10 := IsardPoint milieuDe: Segment2.
	Point10 nom: 'M2'.
	Segment3 := IsardSegment joignant: Point7 avec: Point2.
	Segment3 couleur: ColorValue red.
	Point11 := IsardPoint milieuDe: Segment3.
	Point11 nom: 'M3'.
	Segment4 := IsardSegment joignant: Point7 avec: Point8.
	Segment4 visibilite: false.
	Point12 := IsardPoint milieuDe: Segment4.
	Point12 nom: 'A'''.
	Segment5 := IsardSegment joignant: Point3 avec: Point7.
	Segment5 visibilite: false.
	Point13 := IsardPoint milieuDe: Segment5.
	Point13 nom: 'E'''.
	Segment6 := IsardSegment joignant: Point3 avec: Point8.
	Segment6 visibilite: false.
	Point14 := IsardPoint milieuDe: Segment6.
	Point14 nom: 'F'''.
	Droite5 := IsardDroite passantPar: Point9 etPar: Point10.
	Droite5 couleur: ColorValue darkGreen.
	Droite6 := IsardDroite passantPar: Point9 etPar: Point13.
	Droite6 couleur: ColorValue purple.
	Droite7 := IsardDroite passantPar: Point10 etPar: Point12.
	Droite7 couleur: ColorValue purple.
	Droite8 := IsardDroite passantPar: Point11 etPar: Point12.
	Droite8 couleur: ColorValue purple.
	^plan!

paraboleSauve
	"Executer la commande suivante pour editer l'objet:
	IsardPlanDisplay
		plan: IsardTests paraboleSauve
		limites: ((-81/4)@15 corner: (81/4)@-15)
	"

	| plan Controle1 Point1 Controle2 Point2 Segment1 Controle3 Point3 Controle4 Point4 Segment2 Droite1 Droite2 Droite3 Point5 Lieu1 | 
	plan :=  IsardPlan new. plan nomVar: 'plan'.
	Controle1 := IsardControleCoord p: (169/16)@(-97/16) plan: plan.
	Point1 := IsardPoint coord: Controle1.
	Point1 nom: 'A'.
	Controle2 := IsardControleCoord p: (-25/2)@(-97/16) plan: plan.
	Point2 := IsardPoint coord: Controle2.
	Point2 nom: 'B'.
	Segment1 := IsardSegment joignant: Point1 avec: Point2.
	Segment1 couleur: ColorValue darkGreen.
	Controle3 := IsardControleCoord p: (-1/4)@(-31/16) plan: plan.
	Point3 := IsardPoint coord: Controle3.
	Point3 nom: 'F'.
	Controle4 := IsardControleNombre n: 0.728997 plan: plan.
	Point4 := IsardPoint surLeSegment: Segment1 controlePar: Controle4.
	Point4 nom: 'H'.
	Segment2 := IsardSegment joignant: Point4 avec: Point3.
	Droite1 := IsardDroite mediatriceDe: Segment2.
	Droite1 couleur: ColorValue red.
	Droite2 := IsardDroite passantPar: Point2 etPar: Point1.
	Droite2 visibilite: false.
	Droite3 := IsardDroite perpendiculaireA: Droite2 passantPar: Point4.
	Point5 := IsardPoint intersectionDeLaDroite: Droite3 avecLaDroite: Droite1.
	Point5 nom: 'P'.
	Lieu1 := IsardLieu duPoint: Point5 controlePar: Controle4.
	Lieu1 couleur: ColorValue royalBlue.
	^plan!

problemeManuSauve
	"Executer la commande suivante pour editer l'objet:
	IsardPlanDisplay
		plan: IsardTests problemeManuSauve
		limites: (-9.85@5.75 corner: 6.35@-6.25)
	"

	| plan Coordonne1 Point1 Coordonne2 Point2 Coordonne3 Point3 Coordonne4 Point4 Coordonne5 Point5 Cercle1 Nombre1 Point6 Rotation1 Point7 Nombre2 Point8 Rotation2 Point9 Segment1 Point10 Segment2 Segment3 Segment4 Segment5 | 
	plan :=  IsardPlan new. plan nomVar: 'plan'.
	Coordonne1 := IsardControleCoord p: (-157/40)@(43/20) plan: plan.
	Point1 := IsardPoint coord: Coordonne1.
	Point1 nom: 'A'''.
	Coordonne2 := IsardControleCoord p: (3/4)@(69/40) plan: plan.
	Point2 := IsardPoint coord: Coordonne2.
	Point2 nom: 'B'''.
	Coordonne3 := IsardControleCoord p: -1.65@-0.9 plan: plan.
	Point3 := IsardPoint coord: Coordonne3.
	Point3 nom: 'D'.
	Coordonne4 := IsardControleCoord p: (-287/40)@(-27/8) plan: plan.
	Point4 := IsardPoint coord: Coordonne4.
	Point4 nom: 'O'.
	Coordonne5 := IsardControleCoord p: (-239/40)@(-167/40) plan: plan.
	Point5 := IsardPoint coord: Coordonne5.
	Point5 nom: 'R'.
	Cercle1 := IsardCercle deCentre: Point4 passantPar: Point5.
	Nombre1 := IsardControleNombre n: 0.25 plan: plan.
	Point6 := IsardPoint surLeCercle: Cercle1 controlePar: Nombre1.
	Point6 nom: 'alpha'.
	Rotation1 := IsardRotation deCentre: Point2 dAngle: Nombre1.
	Rotation1 nom: 'RB''-90'.
	Point7 := IsardPoint transformationDe: Point3 par: Rotation1.
	Point7 nom: 'B'.
	Nombre2 := IsardControleNombre n: 0.75 plan: plan.
	Point8 := IsardPoint surLeCercle: Cercle1 controlePar: Nombre2.
	Point8 nom: 'alpha'''.
	Rotation2 := IsardRotation deCentre: Point1 dAngle: Nombre2.
	Rotation2 nom: 'RA''+90'.
	Point9 := IsardPoint transformationDe: Point3 par: Rotation2.
	Point9 nom: 'A'.
	Segment1 := IsardSegment joignant: Point9 avec: Point7.
	Segment1 couleur: ColorValue red.
	Point10 := IsardPoint milieuDe: Segment1.
	Point10 epaisseur: 3 ; nom: 'T'.
	Segment2 := IsardSegment joignant: Point9 avec: Point1.
	Segment2 couleur: ColorValue purple.
	Segment3 := IsardSegment joignant: Point1 avec: Point3.
	Segment3 couleur: ColorValue purple.
	Segment4 := IsardSegment joignant: Point3 avec: Point2.
	Segment4 couleur: ColorValue orange.
	Segment5 := IsardSegment joignant: Point2 avec: Point7.
	Segment5 couleur: ColorValue orange.
	^plan!

triMed
	"Executer la commande suivante pour editer l'objet:
	IsardPlanDisplay
		plan: IsardTests triMed
		limites: ((-81/4)@15 corner: (81/4)@-15)
	"

	| plan Coordonne1 Point1 Coordonne2 Point2 Coordonne3 Point3 Coordonne4 Point4 Droite1 Droite2 Droite3 Vecteur1 Vecteur2 Vecteur3 Nombre1 Transformation1 Point5 Droite4 Transformation2 Point6 Transformation3 Point7 Transformation4 Point8 Segment1 Segment2 Segment3 Cercle1 | 
	plan :=  IsardPlan new. plan nomVar: 'plan'.
	Coordonne1 := IsardControleCoord p: (-53/16)@(1/8) plan: plan.
	Point1 := IsardPoint coord: Coordonne1.
	Point1 nom: 'O'.
	Coordonne2 := IsardControleCoord p: (87/16)@(9/2) plan: plan.
	Point2 := IsardPoint coord: Coordonne2.
	Point2 nom: 'D2'.
	Coordonne3 := IsardControleCoord p: -10@8 plan: plan.
	Point3 := IsardPoint coord: Coordonne3.
	Point3 nom: 'D3'.
	Coordonne4 := IsardControleCoord p: (-27/4)@(-133/16) plan: plan.
	Point4 := IsardPoint coord: Coordonne4.
	Point4 nom: 'D1'.
	Droite1 := IsardDroite passantPar: Point1 etPar: Point4.
	Droite2 := IsardDroite passantPar: Point3 etPar: Point1.
	Droite3 := IsardDroite passantPar: Point2 etPar: Point1.
	Vecteur1 := IsardVecteur joignant: Point1 avec: Point2.
	Vecteur1 couleur: ColorValue red ; epaisseur: 2.
	Vecteur2 := IsardVecteur joignant: Point1 avec: Point3.
	Vecteur2 couleur: ColorValue red ; epaisseur: 2.
	Vecteur3 := IsardVecteur joignant: Point1 avec: Point4.
	Vecteur3 couleur: ColorValue red ; epaisseur: 2.
	Nombre1 := IsardAngleDeVecteurs angleEntre: Vecteur1 et: Vecteur2.
	Transformation1 := IsardRotation deCentre: Point1 dAngle: Nombre1.
	Point5 := IsardPoint transformationDe: Point4 par: Transformation1.
	Point5 nom: 'D'.
	Droite4 := IsardDroite passantPar: Point1 etPar: Point5.
	Transformation2 := IsardSymetrieCentrale deCentre: Point1.
	Point6 := IsardPoint transformationDe: Point5 par: Transformation2.
	Point6 nom: 'B'.
	Transformation3 := IsardSymetrieAxiale dAxe: Droite2.
	Point7 := IsardPoint transformationDe: Point6 par: Transformation3.
	Point7 nom: 'A'.
	Transformation4 := IsardSymetrieAxiale dAxe: Droite1.
	Point8 := IsardPoint transformationDe: Point6 par: Transformation4.
	Point8 nom: 'C'.
	Segment1 := IsardSegment joignant: Point8 avec: Point7.
	Segment1 couleur: ColorValue darkGreen ; epaisseur: 2.
	Segment2 := IsardSegment joignant: Point6 avec: Point7.
	Segment2 couleur: ColorValue darkGreen ; epaisseur: 2.
	Segment3 := IsardSegment joignant: Point6 avec: Point8.
	Segment3 couleur: ColorValue darkGreen ; epaisseur: 2.
	Cercle1 := IsardCercle deCentre: Point1 passantPar: Point6.
	Cercle1 couleur: ColorValue royalBlue ; epaisseur: 2.
	^plan!

viergeSauve
	"Executer la commande suivante pour editer l'objet:
	IsardPlanDisplay
		plan: IsardTests viergeSauve
		limites: ((-81/4)@15 corner: (81/4)@-15)
	"

	| plan | 
	plan :=  IsardPlan new. plan nomVar: 'plan'.
	^plan! !

OrderedCollection variableSubclass: #IsardArgumentsMethodeDeCreation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardArgumentsMethodeDeCreation comment:
'Arguments d''une methode de creation (OrderedCollection de classes)'!


!IsardArgumentsMethodeDeCreation methodsFor: 'compatibilite'!

estCompatibleAvec: selection 
	"Determine si selection peut etre passe comme arguments "
	"une mthode dont les besoins en argument sont dfinis par self"
	"il faut: meme nb d'args"
	"et chaque arg class includesBehavior: classeMinimaleVoulue"

	self size == selection size ifFalse: [^false].
	self with: selection do: [:mine :other | (other class includesBehavior: mine)
			ifFalse: [^false]].
	^true! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardArgumentsMethodeDeCreation class
	instanceVariableNames: ''!


!IsardArgumentsMethodeDeCreation class methodsFor: 'instance creation'!

avecPourNoms: noms 
	"cre une OrderedCollection de classes a partir"
	"d'une collection (un array ici) des noms des classes"

	| aMDC |
	aMDC := self new.
	noms do: [:n | aMDC add: (Smalltalk at: n)].
	^aMDC! !

ControllerWithMenu subclass: #IsardController
	instanceVariableNames: 'objetSuivi handPosition '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardController comment:
'Controller de la IsardView

Instance Variables:
	handPosition	<Point>	position de la main dans les coordonnees du modele quand on pan (ctrl+drag).
	objetSuivi	<IsardObjet>	objet draggu (quand c''est le cas), l''IsardObjet en question est deplacable

'!


!IsardController methodsFor: 'actions du menu'!

choisitCouleur: c 
	model selectionCouleur: c!

choisitCree: objet 
	"l'objet est deja cr"

	model deselectionneTout.
	objet isNil ifTrue: [^self].
	objet selectionne!

choisitCreeMacro
	| n c |
	c := false.
	n := Dialog
				request: 'Nom de la Macro...'
				initialAnswer: 'Test'
				onCancel: [c := true].
	c ifTrue: [^self].
	model sauve: model selection commeMacroNommee: n!

choisitCreePointEn: p 
	model deselectionneTout.
	(model creePointEn: p) selectionne.
	self choisitRenomme!

choisitEffaceTout
	model effaceTout!

choisitEnleve
	model enleveSelection.
	model nettoie!

choisitEpaisseur: e 
	model selectionEpaisseur: e!

choisitImprime
	| n |
	n := Dialog request: 'Nom du fichier...' initialAnswer: 'sortie.eps'.
	n isEmpty ifFalse: [view imprimerDansLeFichier: n]!

choisitInspect
	model selection isEmpty
		ifTrue: [model inspect]
		ifFalse: [model selection do: [:o | o inspect]]!

choisitInvisible
	model selectionVisibilite: false!

choisitNettoie
	model nettoie!

choisitRenomme
	| n c |
	model selection isEmpty ifTrue: [^self].
	c := false.
	n := Dialog
				request: 'Nouveau nom...'
				initialAnswer: (model selection first nom isNil
						ifTrue: ['']
						ifFalse: [model selection first nom])
				onCancel: [c := true].
	c ifFalse: [model selectionNom: (n isEmpty ifFalse: [n])]!

choisitSauve
	| selecteur classe |
	classe := Dialog request: 'Classe...' initialAnswer: 'IsardTests'.
	classe isNil ifTrue: [^nil].
	selecteur := Dialog request: 'Selecteur...' initialAnswer: ''.
	classe := Smalltalk at: classe
				ifAbsent: 
					["erreur"

					^nil].
	selecteur isEmpty 
		ifFalse: 
			[model 
				sauvegardeVersLeSelecteur: selecteur
				deLaClasse: classe
				avecLimites: view limites]!

choisitSubstitue
	model selectionSubstitue.
	model nettoie!

choisitVisible
	model selectionVisibilite: true! !

!IsardController methodsFor: 'menu'!

ajouteAuMenu: menuBuilder listeObjets: liste 
	| dico |
	dico := Dictionary new.
	liste do: [:obj | (dico at: obj nomType ifAbsentPut: [OrderedCollection new])
			add: obj].
	dico
		keysAndValuesDo: 
			[:type :objs | 
			menuBuilder beginSubMenuLabeled: type.
			objs do: [:obj | menuBuilder add: obj -> [:r | obj inverseSelection]].
			menuBuilder endSubMenu]!

initMenuEn: aPoint
	"bon: ca avait une bonne tete mais une -bip- de manip a fait un format"
	"alors maintenant je refuse de le remettre de facon que ca soit lisible"

	| mb m nc p |
	p := self view invTrans: aPoint.
	mb := MenuBuilder new.
	mb 
	add: 'About...' -> [:r | Dialog warn: 'Isard, (c) J.S. Roy 1996-1999
email: jean-sebastien.roy@wanadoo.fr'];
	line;
	add: 'Zoom in' -> [:r | self view centre: p. r view zoom: 2];
	add: 'Zoom out' -> [:r | self view centre: p. r view zoom: 0.5];
	line; 
	add: 'Cr point' -> [:r | r choisitCreePointEn: p];
	beginSubMenuLabeled: 'Cre'.
		nc := IsardObjet dictionnaireMethodesDeCreation menuDeCreationPourLaSelection: model selection ajouteAuMenu: mb.
	mb endSubMenu; 
	add: 'Renomme...' ->  #choisitRenomme;
	add: 'Substitue' -> #choisitSubstitue;
	add: 'Inspect' -> #choisitInspect;
	beginSubMenuLabeled: 'Nom(s) d''origine'.
		model selection do: [:obj | mb add: (obj nomSpecifique) -> nil].
	mb endSubMenu; 
	line;
	beginSubMenuLabeled: 'Couleur'.
		ColorValue constantNames do: [:c | mb add: c -> [:r | r choisitCouleur: (ColorValue perform: c)]].
	mb endSubMenu; 
	beginSubMenuLabeled: 'Epaisseur'.
		1 to: 5 do: [:n | mb add: n printString -> [:r | r choisitEpaisseur: n]].
	mb endSubMenu; 
	beginSubMenuLabeled: 'Visibilit'; 
		add: 'Visible' -> #choisitVisible; 
		add: 'Invisible' -> #choisitInvisible; 
	endSubMenu; 
	line;
	beginSubMenuLabeled: 'Selectionne'.
		self ajouteAuMenu: mb listeObjets: model horsSelection.
	mb endSubMenu; beginSubMenuLabeled: 'Deselectionne'.
		self ajouteAuMenu: mb listeObjets: model selection.
	mb endSubMenu; 
	line;
	add: 'Sauve...' -> #choisitSauve;
	add: 'Imprime...' ->#choisitImprime;
	add: 'Cr Macro...' ->#choisitCreeMacro;
	line;
	add: 'Nettoie' -> #choisitNettoie;
	add: 'Enlve' -> #choisitEnleve;
	add: 'Efface tout' -> #choisitEffaceTout.
	m :=mb menu.
	model selection isEmpty ifTrue: [
		(m menuItemLabeled: 'Couleur') disable.
		(m menuItemLabeled: 'Epaisseur') disable.
		(m menuItemLabeled: 'Visibilit') disable.
		(m menuItemLabeled: 'Deselectionne') disable.
		(m menuItemLabeled: 'Enlve') disable.
		(m menuItemLabeled: 'Cr Macro...') disable.
		(m menuItemLabeled: 'Nom(s) d''origine') disable.
	].
	model selection size ~= 2 ifTrue: [
		(m menuItemLabeled: 'Substitue') disable.
	] ifFalse: [
		model selectionEstSubstituable ifFalse: [
			(m menuItemLabeled: 'Substitue') disable.
		]
	].
	nc isZero ifTrue: [
		(m menuItemLabeled: 'Cre') disable.
	].
	model horsSelection isEmpty ifTrue: [
		(m menuItemLabeled: 'Selectionne') disable.
	].
	model inutiles isEmpty ifTrue: [
		(m menuItemLabeled: 'Nettoie') disable.
	].
	self menuHolder: m asValue.
	self performer: self! !

!IsardController methodsFor: 'selection'!

choisitObjetParmis: liste en: aPoint 
	| mb menu result |
	liste isEmpty ifTrue: [^nil].
	liste size == 1 ifTrue: [^liste first].
	self sensor windowSensor queueLocked: false.
	mb := MenuBuilder new.
	liste do: [:o | mb add: o nomParDefaut -> o].
	self menuHolder: mb menu asValue.
	menu := self menu.
	result := self
				trackMenu: menu
				at: aPoint
				centered: false.
	^result!

objetsAutourDe: pt aMoinsDe: dt 
	"pt est un Point dans les coordonnees de la vue (pas du modele)"
	"et dt est une distance dans les coordonnees de la vue"

	| p liste cd d |
	p := view invTrans: pt.
	d := view invDistTrans: dt.
	liste := SortedCollection new.
	self model descendance do: [:o | o estGeometrique & o visibilite & o validite
			ifTrue: 
				[cd := o distanceAuPoint: p.
				o estDeplacable ifTrue: [cd := cd - (view invDistTrans: 10)].
				cd <= d ifTrue: [liste add: cd -> o]]].
	^liste collect: [:o | o value]! !

!IsardController methodsFor: 'event driven'!

enterEvent: event 
	Cursor crossHair show!

exitEvent: anExitEvent 
	Cursor normal show!

mouseMovedEvent: event 
	| p ep |
	event redButtonPressed ifFalse: [^self].
	ep := self sensor mousePointFor: event.
	p := view invTrans: ep.
	event ctrlDown
		ifTrue: 
			[handPosition isNil ifTrue: [handPosition := p].
			Cursor hand show.
			self view faitCorrespondre: handPosition avec: ep]
		ifFalse: 
			[handPosition := nil.
			objetSuivi isNil ifTrue: [^self].
			Cursor hand show.
			objetSuivi vaEn: p]!

redButtonPressedEvent: event 
	| liste objet p ep |
	ep := self sensor mousePointFor: event.
	liste := self objetsAutourDe: ep aMoinsDe: 20.
	objetSuivi := nil.
	p := self view invTrans: ep.
	event ctrlDown 
		ifTrue: 
			[Cursor hand show.
			handPosition := p]
		ifFalse: 
			[event shiftDown 
				ifTrue: 
					[objet := self choisitObjetParmis: liste en: event globalPoint.
					objet isNil 
						ifFalse: 
							[objet selectionnite 
								ifTrue: [objet deselectionne]
								ifFalse: [objet selectionne]]]
				ifFalse: 
					[((model selection size = 1 and: [model selection first estDeplacable]) 
						and: [liste includes: model selection first]) 
							ifTrue: 
								[Cursor hand show.
								objetSuivi := model selection first.
								objetSuivi vaEn: p]
							ifFalse: 
								[liste isEmpty 
									ifTrue: [model deselectionneTout]
									ifFalse: 
										[objet := self choisitObjetParmis: liste en: event globalPoint.
										objet isNil 
											ifFalse: 
												[model deselectionneTout.
												objet selectionne]]]]]!

redButtonReleasedEvent: event 
	Cursor crossHair show.
	handPosition := nil!

yellowButtonPressedEvent: event 
	| ep |
	ep := self sensor mousePointFor: event.
	self initMenuEn: ep.
	super yellowButtonPressedEvent: event! !

Model subclass: #IsardObjet
	instanceVariableNames: 'filsDirects descendance parents validite visibilite nom couleur epaisseur monPlan selectionnite nomVar '
	classVariableNames: 'DicoDMDC '
	poolDictionaries: ''
	category: 'Isard'!
IsardObjet comment:
'Objet gometrique.
fourni tous les services communs aux objets gometriques.

Subclasses must implement the following messages:
	deplacement (si estDeplacable)
		vaEn:
	distance-projection (si estGeometrique)
		projeteDuPoint:

Instance Variables:
	couleur	<ColorValue>	couleur de l''objet
	descendance	<OrderedCollection>	descendance trie par ordre de creation de l''objet
	epaisseur	<Number>	epaisseur de l''objet
	filsDirects	<OrderedCollection>	fils immediats de l''objet
	monPlan	<IsardPlan>	plan qui contient l''objet
	nom	<String>	nom de l''objet (ou nil)
	nomVar	<String>	nom de la variable contenant l''objet lors de la sauvegarde (ou nil)
	parents	<OrderedCollection>	parents de l''objet
	selectionnite	<Boolean>	est selectionn ?
	validite	<Boolean>	validit
	visibilite	<Boolean>	visibilit

Class Variables:
	DicoDMDC	<IsardDictionnaireMethodesDeCreation>	dictionnaire contenant l''ensemble des methodes de creations utilisables pour creer des objets gomtriques

'!


!IsardObjet methodsFor: 'style'!

affecte: valeur aAttribut: selecteur 
	self perform: (selecteur , ':') asSymbol with: valeur!

couleur
	^couleur!

couleur: c 
	couleur := c.
	self traceAChange!

couleurInitial
	^ColorValue black!

epaisseur
	^epaisseur!

epaisseur: e 
	epaisseur := e.
	self traceAChange!

epaisseurInitial
	^1!

invisible
	visibilite := false.
	self traceAChange!

valeurAttribut: selecteur 
	^self perform: selecteur!

valeurInitialeAttribut: selecteur 
	^self perform: (selecteur , 'Initial') asSymbol!

visibilite
	^visibilite!

visibilite: v 
	visibilite := v.
	self traceAChange!

visibiliteInitial
	^true!

visible
	visibilite := true.
	self traceAChange! !

!IsardObjet methodsFor: 'nomage'!

autoNomme
	nom notNil ifTrue: [^self].
	nom := self nomVar!

nom
	^nom!

nom: n 
	nom := n.
	self traceAChange!

nomInitial
	^nil!

nomParDefaut
	^self nom isNil
		ifTrue: [self nomSpecifique]
		ifFalse: [self nom]!

nomSpecifique
	^self class nomSpecifiqueAvecParents: parents!

nomType
	^self class nomType!

nomTypePrecis
	^self class nomTypePrecis! !

!IsardObjet methodsFor: 'selection'!

deselectionne
	selectionnite := false.
	monPlan deselectionne: self.
	self traceAChange!

inverseSelection
	self selectionnite: selectionnite not!

selectionne
	selectionnite := true.
	monPlan selectionne: self.
	self traceAChange!

selectionnite
	^selectionnite!

selectionnite: anObject 
	selectionnite := anObject.
	selectionnite
		ifTrue: [monPlan selectionne: self]
		ifFalse: [monPlan deselectionne: self].
	self traceAChange!

selectionniteInitial
	^false! !

!IsardObjet methodsFor: 'accessing'!

descendance
	^descendance!

estGeometrique
	"renvoie true si l'objet a une reelle consistance sur le papier"

	^false!

filsDirects
	^filsDirects!

monPlan
	^monPlan!

monPlan: anIsardPlan 
	monPlan := anIsardPlan!

parent1
	^parents at: 1!

parent2
	^parents at: 2!

parent3
	^parents at: 3!

parents
	^parents!

parents: p 
	parents := p! !

!IsardObjet methodsFor: 'initialisation'!

init
	"initialisation d'un objet (c'est le minimum)"
	"attention les attributs ne sont pas affects"

	filsDirects := OrderedCollection new.
	descendance := OrderedCollection new.
	parents := OrderedCollection new!

postInit
	"initialisation d'un objet apres avoir mis des parents etc"
	"si la sous classe en a besoin"

	self listeDesAttributs do: [:m | self affecte: (self valeurInitialeAttribut: m)
			aAttribut: m]! !

!IsardObjet methodsFor: 'famille'!

ajouteDescendance: d 
	"ajoute une collection d'objets a la descendance"

	| n |
	n := d reject: [:i | descendance includes: i].
	self ajouteNouvelleDescendance: n.
	parents do: [:i | i ajouteDescendance: n]!

ajouteFils: f 
	| g |
	filsDirects add: f.
	(descendance includes: f)
		ifTrue: [^self].
	g := OrderedCollection new.
	g add: f.
	g addAll: f descendance.
	self ajouteDescendance: g!

ajouteNouvelleDescendance: d 
	"idem ajouteDescendance, mais les objets ne sont pas deja dans la 
	descendance"

	descendance addAll: d!

enleve
	"Supprime un objet. Une fois supprim il ne doit PLUS etre rfrenc."
	"on elimine tous les objets de la descendance (a reculons), puis on s'enleve 
	soi-meme"

	[descendance isEmpty]
		whileFalse: [descendance last enleve].
	parents do: [:i | i enleveFils: self].
	self traceAChange!

enleveDescendance: f 
	(descendance includes: f)
		ifFalse: [^self].
	descendance remove: f.
	parents do: [:p | p enleveDescendance: f]!

enleveFils: f 
	"supprime un fils"

	filsDirects remove: f.
	self enleveDescendance: f!

estSubstituablePar: r 
	"test simple de coherence, mais insuffisant vis a vis d'une utilisation interactive"

	filsDirects do: [:o | (o methodeDeCreation arguments estCompatibleAvec: (o parents collect: [:i | i == self
					ifTrue: [r]
					ifFalse: [i]]))
			ifFalse: [^false]].
	^true!

influantsSurSoiQuandOnChange: p 
	"determine l'ensemble des objets a mettre a jour pour"
	"se mettre a jour quand on modifie p."

	| t d |
	t := self tousParents.
	t add: self.
	d := p descendance.
	^d select: [:e | t includes: e]!

inspectFils
	"outil de debug, inspecte chaque fils"

	filsDirects do: [:f | f inspect]!

substituePar: r 
	"tres^42 risqu"
	"passe outre 99% des routines de bases"
	"est bas sur un fonctionement TRES particulier des accesseurs (no-copy)"

	| p |
	parents
		do: 
			[:o | 
			o enleveFils: self.
			descendance do: [:d | o enleveDescendance: d]].
	r filsDirects addAll: filsDirects.
	r ajouteDescendance: descendance.
	filsDirects
		do: 
			[:o | 
			p := o parents.
			1 to: p size do: [:i | (p at: i)
					== self ifTrue: [p at: i put: r]]].
	r aChange!

tousParents
	"determine l'ensemble des parents/grands-parents etc. d'un objet"
	"tres inefficace."

	| t |
	t := IdentitySet new.
	t addAll: parents.
	parents do: [:p | t addAll: p tousParents].
	^t! !

!IsardObjet methodsFor: 'mise a jour'!

aChange
	"ce message est envoy par un objet qui a ete modifi: il a Chang"
	"il mets en conscequence a jour ses descendants"

	descendance do: [:i | i miseAJour].
	self traceAChange!

change
	"ce message est recu par un objet pour le lequel un parent a chang"
	"il se mets donc a jour"!

miseAJour
	"si tous les parents sont valides ont envoie 'change' a l'objet"
	"sinon il est invalide"

	(parents contains: [:p | p validite not])
		ifTrue: [self invalide]
		ifFalse: 
			[self valide.
			self change]!

traceAChange
	monPlan changed: #trace! !

!IsardObjet methodsFor: 'validation'!

invalide
	validite := false!

valide
	validite := true!

validite
	^validite!

validite: v 
	validite := v!

validiteInitial
	^true! !

!IsardObjet methodsFor: 'trace'!

aRetracerPour: v sur: gc 
	validite ifFalse: [^self].
	self retraceCompletPour: v sur: gc.
	descendance do: [:f | f retraceCompletPour: v sur: gc]!

retraceCompletPour: v sur: gc 
	validite ifTrue: [visibilite
			ifTrue: 
				[gc paint: couleur.
				gc lineWidth: epaisseur + (selectionnite
							ifTrue: [2]
							ifFalse: [0]).
				self retraceNomPour: v sur: gc.
				self retracePour: v sur: gc]]!

retraceNomPour: v sur: gc!

retracePour: v sur: gc! !

!IsardObjet methodsFor: 'sauvegarde'!

listeDesAttributs
	^#(#couleur #epaisseur #visibilite #nom #selectionnite #validite)!

messageDeCreation
	"renvoie un message creant l'objet"

	^self messageDeCreationAvecParentsNommes: (parents collect: [:o | o nomVar])!

messageDeCreationAvecParentsNommes: npar 
	"renvoie un message creant l'objet sachant que les parents ont pour noms les 
	elements de npar"
	"les parametres DOIVENT etre exprimables via printString"

	| opar |
	opar := OrderedCollection new.
	self class parametres do: [:p | opar add: (self perform: p first) printString].
	opar addAll: npar.
	^self methodeDeCreation messageDeCreationAvecParametresNommes: opar!

messageDeSauvegarde
	"renvoie un message creant l'objet et l'affectant"
	"utiliser plutot un Stream"

	| s f c |
	f := self nomVar , ' := ' , self messageDeCreation.
	s := WriteStream on: String new.
	nomVar notNil ifTrue: [s nextPutAll: ' ; nomVar: '; nextPutAll: self nomVar printString].
	self listeDesAttributs do: [:m | (self valeurAttribut: m)
			~= (self valeurInitialeAttribut: m) ifTrue: [s nextPutAll: ' ; '; nextPutAll: m; nextPutAll: ': '; nextPutAll: (self valeurAttribut: m) printString]].
	c := s contents.
	c isEmpty ifFalse: [c := '
	' , self nomVar , (c copyFrom: 3 to: c size) , '.'].
	^f , c!

methodeDeCreation
	"renvoie la 'methode de creation' qui permet de regenerer l'objet"

	^(IsardObjet dictionnaireMethodesDeCreation at: self parents size + self class parametres size ifAbsent: [^nil])
		detect: [:mdc | mdc classeCree == self class]
		ifNone: nil!

nomVar
	^nomVar isNil
		ifTrue: [self nomType , ((monPlan descendance select: [:o | o nomType == self nomType])
					indexOf: self) printString]
		ifFalse: [nomVar]!

nomVar: n 
	nomVar := n!

sauve: objets commeMacroNommee: nomMacro 
	| s classe classeSymbol mdc desc desc1 |
	desc1 := OrderedCollection new.
	objets do: [:o | o descendance do: [:d | (desc1 includes: d)
				ifFalse: [desc1 add: d]]].
	desc := self monPlan descendance select: [:e | desc1 includes: e].
	desc isEmpty ifTrue: [self error: 'Macro dbile. Essaye encore.'].
	desc do: [:d | d parents do: [:p | (objets includes: p)
				| (desc includes: p) ifFalse: [self error: 'Macro dbile. Essaye encore.']]].
	classeSymbol := ('IsardMacro' , nomMacro) asSymbol.
	(Smalltalk includesKey: classeSymbol)
		ifTrue: [self error: 'la classe ' , classeSymbol printString , ' existe dja.'].
	IsardMacro
		subclass: classeSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'Isard'.
	classe := Smalltalk at: classeSymbol.
	classe class compile: 'nomTypePrecis
	^''' , nomMacro , '''' classified: #nomage.
	mdc := 'methodeDeCreation
	^#(#'.
	objets do: [:o | mdc := mdc , 'arg:'].
	mdc := mdc , '  #('.
	objets do: [:o | mdc := mdc , o class name asSymbol printString , ' '].
	mdc := mdc , ' ))'.
	classe class compile: mdc classified: 'methode de creation' asSymbol.
	s := WriteStream on: String new.
	objets do: [:o | s nextPutAll: 'arg: '; nextPutAll: o nomVar; nextPutAll: ' '].
	s crtab; nextPutAll: '| '.
	desc do: [:o | s nextPutAll: o nomVar; nextPutAll: ' '].
	s nextPutAll: '| '; crtab.
	desc do: [:o | s nextPutAll: o messageDeSauvegarde; crtab].
	s nextPutAll: '^nil'.
	classe class compile: s contents classified: 'instance creation' asSymbol.
	Transcript cr; show: classe class name , ' defined'.
	IsardObjet initialize! !

!IsardObjet methodsFor: 'printing'!

asString
	^self nomParDefaut!

printOn: aStream 
	aStream nextPutAll: self messageDeCreation! !

!IsardObjet methodsFor: 'distance-projection'!

distanceAuPoint: p 
	"renvoie la distance d'un point p a soit-meme"

	^p dist: (self projeteDuPoint: p)!

projeteDuPoint: p 
	"renvoie le projet du point p sur soit-mme"
	"est implement uniquement pour les classes 'gometriques'"

	self estGeometrique
		ifTrue: [self subclassResponsibility]
		ifFalse: [self error: 'cet IsardObjet n''est pas gometrique']! !

!IsardObjet methodsFor: 'deplacement'!

estDeplacable
	^false!

vaEn: aPoint 
	self estDeplacable
		ifTrue: [self subclassResponsibility]
		ifFalse: [self error: 'cet IsardObjet n''est pas deplacable']! !

!IsardObjet methodsFor: 'calculs'!

calcule
	^true! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardObjet class
	instanceVariableNames: ''!


!IsardObjet class methodsFor: 'instance creation'!

new
	"new+initialisation"

	^super new init postInit!

newAvecParent: p 
	| g |
	g := OrderedCollection new.
	g add: p.
	^self newAvecParents: g!

newAvecParent: p1 etParent: p2 
	| g |
	g := OrderedCollection new.
	g add: p1.
	g add: p2.
	^self newAvecParents: g!

newAvecParent: p1 etParent: p2 etParent: p3 
	| g |
	g := OrderedCollection new.
	g add: p1.
	g add: p2.
	g add: p3.
	^self newAvecParents: g!

newAvecParents: p 
	"cree un nouvel objets avec p comme parents."
	"chaque parent le recoit comme fils, et il se mets a jour"
	"ainsi que sa descendance"

	| o |
	o := self newSansPostInit parents: p.
	p do: [:i | i ajouteFils: o].
	o postInit.
	o miseAJour.
	^o aChange!

newSansPostInit
	"new+initialisation"

	^super new init! !

!IsardObjet class methodsFor: 'resources'!

ImageMotifInvisible
	"UIMaskEditor new openOnClass: self andSelector: #ImageMotifInvisible"

	<resource: #image>
	^CachedImage on: (Image extent: 2@2 depth: 1 bitsPerPixel: 1 palette: (MappedPalette withColors: ((Array new: 2) at: 1 put: ColorValue red; at: 2 put: ColorValue white; yourself)) usingBits: #[64 0 0 0 128 0 0 0])!

MotifInvisible
	self ImageMotifInvisible asPattern! !

!IsardObjet class methodsFor: 'dictionnaire des methodes de creation'!

dictionnaireMethodesDeCreation
	DicoDMDC isNil ifTrue: [DicoDMDC := IsardDictionnaireMethodesDeCreation new].
	^DicoDMDC!

enregistreMethodeDeCreation: mdc 
	self dictionnaireMethodesDeCreation ajouteMethodeDeCreation: mdc!

reinitialiseDictionnaireMethodesDeCreation
	DicoDMDC := nil! !

!IsardObjet class methodsFor: 'parametres'!

parametres
	^#()! !

!IsardObjet class methodsFor: 'methode de creation'!

methodeDeCreation
	^nil! !

!IsardObjet class methodsFor: 'class initialization'!

initialise
	"initialise une mdc"

	| t mdc |
	t := self methodeDeCreation.
	t isNil ifTrue: [^self].
	mdc := IsardMethodeDeCreation new.
	mdc selecteur: (t at: 1).
	mdc arguments: (IsardArgumentsMethodeDeCreation avecPourNoms: (t at: 2)).
	mdc classeCree: self.
	IsardObjet enregistreMethodeDeCreation: mdc!

initialize
	"initialize integralement le dictionnaire des mdc"

	IsardObjet reinitialiseDictionnaireMethodesDeCreation.
	(((Smalltalk organization listAtCategoryNamed: #Isard)
		collect: [:cn | Smalltalk at: cn])
		select: [:c | c class selectors includes: #methodeDeCreation])
		do: [:c | c initialise]! !

!IsardObjet class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	| n |
	n := self nomTypePrecis.
	parents isEmpty
		ifFalse: 
			[n := n , '('.
			parents do: [:p | n := n , p nomParDefaut , (parents last == p
								ifTrue: [')']
								ifFalse: [','])]].
	^n!

nomType
	^'Objet'!

nomTypePrecis
	^self nomType! !

IsardObjet subclass: #IsardMacro
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMacro comment:
'Fourni la classe et les services de base pour toutes les macros
'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardMacro class
	instanceVariableNames: ''!


!IsardMacro class methodsFor: 'nomage'!

nomType
	^'Macro'! !

IsardObjet subclass: #IsardVecteur
	instanceVariableNames: 'p v '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardVecteur comment:
'vecteur

Instance Variables:
	p	<Point>	 extrmit (oui je sais pour un vecteur c''est ridicule mais bon)
	v	<Point>	le vecteur a proprement dit.

'!


!IsardVecteur methodsFor: 'distance-projection'!

projeteDuPoint: pt 
	| u t |
	v isZero
		ifTrue: [^p]
		ifFalse: 
			[u := p - pt.
			t := (u dotProduct: v)
						/ (v dotProduct: v) negated.
			t < 0 ifTrue: [^p].
			t > 1 ifTrue: [^p + v].
			^t * v + p]! !

!IsardVecteur methodsFor: 'trace'!

retracePour: vue sur: gc 
	| t |
	t := v y negated @ v x + v / 10.
	gc displayLineFrom: (vue trans: p)
		to: (vue trans: p + v).
	gc displayLineFrom: (vue trans: p + v - t)
		to: (vue trans: p + v).
	gc displayLineFrom: (vue trans: p + v + (t y negated @ t x))
		to: (vue trans: p + v)! !

!IsardVecteur methodsFor: 'accessing'!

estGeometrique
	"renvoie true si l'objet a une reelle consistance sur le papier"

	^true!

p
	^p!

v
	^v! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardVecteur class
	instanceVariableNames: ''!


!IsardVecteur class methodsFor: 'instance creation'!

joignant: p1 avec: p2 
	^IsardVecteurDepuis2Points newAvecParent: p1 etParent: p2! !

!IsardVecteur class methodsFor: 'nomage'!

nomType
	^'Vecteur'! !

IsardObjet subclass: #IsardPlan
	instanceVariableNames: 'selection '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPlan comment:
'Plan

Instance Variables:
	selection	<OrderedCollection>	ensemble des objets slctionns
'!


!IsardPlan methodsFor: 'famille'!

ajouteNouvelleDescendance: d 
	"defini le plan de tout ces nouveaux objets"
	"et se retrace"

	d do: [:f | f monPlan: self].
	super ajouteNouvelleDescendance: d.
	self changed! !

!IsardPlan methodsFor: 'initialisation'!

init
	super init.
	monPlan := self!

postInit
	selection := OrderedCollection new.
	super postInit! !

!IsardPlan methodsFor: 'selection private'!

deselectionne: o 
	"attention n'est appele que par les IsardObjets"

	selection remove: o ifAbsent: []!

selectionne: o 
	"attention n'est appele que par les IsardObjets"

	(selection includes: o)
		ifTrue: [^self].
	selection add: o! !

!IsardPlan methodsFor: 'actions sur la selection'!

enleveSelection
	selection do: [:o | (descendance includes: o)
			ifTrue: [o enleve]].
	selection := OrderedCollection new!

selectionCouleur: c 
	selection do: [:o | o couleur: c]!

selectionEpaisseur: e 
	selection do: [:o | o epaisseur: e]!

selectionEstSubstituable
	^selection first estSubstituablePar: (selection at: 2)!

selectionNom: n 
	selection do: [:o | o nom: n]!

selectionSubstitue
	"substitue le le premier element de la selection par le deuxieme"
	"dangereux maglre les quelques verifications"

	selection first substituePar: (selection at: 2).
	selection remove: selection first!

selectionVisibilite: v 
	selection do: [:o | o visibilite: v]! !

!IsardPlan methodsFor: 'selection'!

deselectionneTout
	[selection isEmpty]
		whileFalse: [selection first deselectionne]!

horsSelection
	^descendance reject: [:o | o selectionnite]!

selection
	^selection!

selectionneTout: c 
	c do: [:o | o selectionne]! !

!IsardPlan methodsFor: 'actions de creation'!

creePointEn: p 
	^(IsardPoint coord: (IsardControleCoord p: p plan: self)) autoNomme! !

!IsardPlan methodsFor: 'sauvegarde'!

blocDeSauvegarde
	| s |
	s := WriteStream on: String new.
	s tab; nextPutAll: '| '; nextPutAll: self nomVar; nextPutAll: ' '.
	descendance do: [:o | s nextPutAll: o nomVar; nextPutAll: ' '].
	s nextPutAll: '| '; crtab; nextPutAll: self messageDeSauvegarde; crtab.
	descendance do: [:o | s nextPutAll: o messageDeSauvegarde; crtab].
	s nextPutAll: '^'; nextPutAll: self nomVar.
	^s contents!

messageDeCreation
	"renvoie un message creant l'objet"

	^' IsardPlan new.'!

messageDeSauvegarde
	"renvoie un message creant l'objet et l'affectant"

	| s |
	s := self nomVar , ' := ' , self messageDeCreation.
	s := s , ' ' , self nomVar , ' nomVar: ' , self nomVar printString , '.'.
	^s!

nomVar
	^nomVar isNil
		ifTrue: ['plan']
		ifFalse: [nomVar]!

sauvegardeVersLeSelecteur: sel deLaClasse: classe avecLimites: l 
	| s |
	s := WriteStream on: String new.
	s nextPutAll: sel; cr.
	s tab; nextPutAll: '"Executer la commande suivante pour editer l''objet:"
	"IsardPlanDisplay'; crtab; tab; nextPutAll: 'plan: '; nextPutAll: classe name; space; nextPutAll: sel; crtab; tab; nextPutAll: 'limites: ('; nextPutAll: l printString; nextPutAll: ')"'; cr; cr; nextPutAll: self blocDeSauvegarde.
	classe class compile: s contents classified: #IsardResources.
	Transcript cr; show: classe class name , '>' , sel , ' defined'! !

!IsardPlan methodsFor: 'actions diverses'!

effaceTout
	self enleve!

inutiles
	^descendance select: [:o | o estGeometrique not & o filsDirects isEmpty]!

nettoie
	self inutiles do: [:i | i enleve]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPlan class
	instanceVariableNames: ''!


!IsardPlan class methodsFor: 'nomage'!

nomType
	^'Plan'! !

IsardObjet subclass: #IsardDroite
	instanceVariableNames: 'a b c '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardDroite comment:
'Droite

Instance Variables:
	a	<Number>
	b	<Number>
	c	<Number>

tels que: ax+by+c=0'!


!IsardDroite methodsFor: 'accessing'!

a
	^a!

b
	^b!

c
	^c!

estGeometrique
	"renvoie true si l'objet a une reelle consistance sur le papier"

	^true!

vecteurNormalUnitaire
	^(a @ b) unitVector! !

!IsardDroite methodsFor: 'trace'!

retracePour: v sur: gc 
	| xl yu xr yd def xu p1 xd p2 o yl yr bo co |
	bo := v bounds.
	o := v invTrans: bo origin.
	co := v invTrans: bo corner.
	xl := o x.
	yu := o y.
	xr := co x.
	yd := co y.
	def := 0.
	a isZero
		ifFalse: 
			[xu := (b * yu + c / a) negated.
			xl <= xu & (xu <= xr)
				ifTrue: 
					[p1 := xu @ yu.
					def := 1].
			xd := (b * yd + c / a) negated.
			xl <= xd & (xd <= xr) ifTrue: [def == 1
					ifTrue: 
						[p2 := xd @ yd.
						def := 2]
					ifFalse: 
						[p1 := xd @ yd.
						def := 1]]].
	b isZero | (def == 2)
		ifFalse: 
			[yl := (a * xl + c / b) negated.
			yd <= yl & (yl <= yu) ifTrue: [def == 0
					ifTrue: 
						[p1 := xl @ yl.
						def := 1]
					ifFalse: 
						[p2 := xl @ yl.
						def := 2]].
			yr := (a * xr + c / b) negated.
			yd <= yr & (yr <= yu) ifTrue: [def == 1
					ifTrue: 
						[p2 := xr @ yr.
						def := 2]
					ifFalse: [def == 2 & (p1 = p2)
							ifTrue: 
								[p2 := xr @ yr.
								def := 2]]]].
	def == 2 ifTrue: [gc displayLineFrom: (v trans: p1)
			to: (v trans: p2)]! !

!IsardDroite methodsFor: 'distance-projection'!

projeteDuPoint: p 
	| u t |
	u := a @ b.
	t := (c + (p dotProduct: u) / (u dotProduct: u)) negated.
	^p + (t * u)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardDroite class
	instanceVariableNames: ''!


!IsardDroite class methodsFor: 'instance creation'!

bissectriceDe: p1 et: p2 et: p3 
	^IsardDroiteBissectrice
		newAvecParent: p1
		etParent: p2
		etParent: p3!

mediatriceDe: s 
	^IsardDroiteMediatrice newAvecParent: s!

passantPar: p1 etPar: p2 
	^IsardDroitePar2Points newAvecParent: p1 etParent: p2!

perpendiculaireA: d passantPar: p 
	^IsardDroitePerpendiculaire newAvecParent: d etParent: p! !

!IsardDroite class methodsFor: 'nomage'!

nomType
	^'Droite'! !

IsardDroite subclass: #IsardDroiteMediatrice
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardDroiteMediatrice comment:
'Mdiatrice d''un segment'!


!IsardDroiteMediatrice methodsFor: 'mise a jour'!

change
	| s v u |
	s := self parent1.
	v := s p2 - s p1.
	u := s p1 + s p2.
	a := v x.
	b := v y.
	c := ((v dotProduct: u)
				/ 2) negated! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardDroiteMediatrice class
	instanceVariableNames: ''!


!IsardDroiteMediatrice class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Mediatrice de ' , (parents at: 1) nomParDefaut!

nomTypePrecis
	^'Mediatrice'! !

!IsardDroiteMediatrice class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#mediatriceDe: #(#IsardSegment))! !

IsardObjet subclass: #IsardCercle
	instanceVariableNames: 'c r '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardCercle comment:
'Cercle

Instance Variables:
	c	<Point>	centre du cercle
	r	<Number>	rayon du cercle

'!


!IsardCercle methodsFor: 'accessing'!

c
	^c!

estGeometrique
	"renvoie true si l'objet a une reelle consistance sur le papier"

	^true!

r
	^r! !

!IsardCercle methodsFor: 'trace'!

retracePour: v sur: gc 
	r * v echelle > 8000 ifTrue: [^self].
	(Circle center: (v trans: c)
		radius: r * v echelle) asStroker displayOn: gc! !

!IsardCercle methodsFor: 'distance-projection'!

projeteDuPoint: p 
	^(Point r: r theta: (p - c) theta)
		+ c! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardCercle class
	instanceVariableNames: ''!


!IsardCercle class methodsFor: 'instance creation'!

deCentre: p etDeRayon: r 
	^IsardCercleDepuisCentreRayon newAvecParent: p etParent: r!

deCentre: p1 passantPar: p2 
	^IsardCercleDepuisCentrePoint newAvecParent: p1 etParent: p2! !

!IsardCercle class methodsFor: 'nomage'!

nomType
	^'Cercle'! !

IsardMacro subclass: #IsardMacroTriangle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMacroTriangle comment:
'macro creant certain elements d''un triangle a partir de 3 points'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardMacroTriangle class
	instanceVariableNames: ''!


!IsardMacroTriangle class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Triangle de sommets ' , (parents at: 1) nomParDefaut , ',' , (parents at: 2) nomParDefaut , ' et ' , (parents at: 3) nomParDefaut , '.'! !

!IsardMacroTriangle class methodsFor: 'instance creation'!

deSommet: pta et: ptb et: ptc 
	| dab dac dbc sab sbc sac ha hb hc ap bp cp a1 b1 c1 h sha shb shc a2 b2 c2 mab mbc mac o cc oh op c9 |
	pta autoNomme.
	ptb autoNomme.
	ptc autoNomme.
	dab := IsardDroite passantPar: pta etPar: ptb.
	dac := IsardDroite passantPar: pta etPar: ptc.
	dbc := IsardDroite passantPar: ptb etPar: ptc.
	sab := IsardSegment joignant: pta avec: ptb.
	sbc := IsardSegment joignant: ptb avec: ptc.
	sac := IsardSegment joignant: ptc avec: pta.
	ha := IsardDroite perpendiculaireA: dbc passantPar: pta.
	hb := IsardDroite perpendiculaireA: dac passantPar: ptb.
	hc := IsardDroite perpendiculaireA: dab passantPar: ptc.
	ha couleur: ColorValue blue.
	hb couleur: ColorValue blue.
	hc couleur: ColorValue blue.
	ap := IsardPoint milieuDe: sbc.
	bp := IsardPoint milieuDe: sac.
	cp := IsardPoint milieuDe: sab.
	ap nom: pta nom , ''''.
	bp nom: ptb nom , ''''.
	cp nom: ptc nom , ''''.
	a1 := IsardPoint intersectionDeLaDroite: ha avecLaDroite: dbc.
	b1 := IsardPoint intersectionDeLaDroite: hb avecLaDroite: dac.
	c1 := IsardPoint intersectionDeLaDroite: hc avecLaDroite: dab.
	a1 nom: pta nom , '1'.
	b1 nom: ptb nom , '1'.
	c1 nom: ptc nom , '1'.
	h := IsardPoint intersectionDeLaDroite: ha avecLaDroite: hb.
	h nom: 'H'.
	sha := IsardSegment joignant: h avec: pta.
	shb := IsardSegment joignant: h avec: ptb.
	shc := IsardSegment joignant: h avec: ptc.
	sha invisible.
	shb invisible.
	shc invisible.
	a2 := IsardPoint milieuDe: sha.
	b2 := IsardPoint milieuDe: shb.
	c2 := IsardPoint milieuDe: shc.
	a2 nom: pta nom , '2'.
	b2 nom: ptb nom , '2'.
	c2 nom: ptc nom , '2'.
	mab := IsardDroite mediatriceDe: sab.
	mbc := IsardDroite mediatriceDe: sbc.
	mac := IsardDroite mediatriceDe: sac.
	mab couleur: ColorValue red.
	mbc couleur: ColorValue red.
	mac couleur: ColorValue red.
	o := IsardPoint intersectionDeLaDroite: mab avecLaDroite: mbc.
	o nom: 'O'.
	cc := IsardCercle deCentre: o passantPar: pta.
	cc couleur: ColorValue royalBlue.
	oh := IsardSegment joignant: o avec: h.
	op := IsardPoint milieuDe: oh.
	op nom: 'O'''.
	c9 := IsardCercle deCentre: op passantPar: a1.
	c9 couleur: ColorValue darkGreen.
	c9 epaisseur: 2.
	^nil! !

!IsardMacroTriangle class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#deSommet:et:et: #(#IsardPoint #IsardPoint #IsardPoint))! !

IsardObjet subclass: #IsardNombre
	instanceVariableNames: 'n '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardNombre comment:
'Nombre

Instance Variables:
	n	<Number>	la valeur

'!


!IsardNombre methodsFor: 'accessing'!

n
	^n! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardNombre class
	instanceVariableNames: ''!


!IsardNombre class methodsFor: 'nomage'!

nomType
	^'Nombre'! !

IsardDroite subclass: #IsardDroitePerpendiculaire
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardDroitePerpendiculaire comment:
'droite perpendiculaire a une autre droite passant par un point'!


!IsardDroitePerpendiculaire methodsFor: 'mise a jour'!

change
	| d p |
	d := self parent1.
	p := self parent2 p.
	a := d b negated.
	b := d a.
	c := (a * p x + (b * p y)) negated! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardDroitePerpendiculaire class
	instanceVariableNames: ''!


!IsardDroitePerpendiculaire class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Perpendiculaire a ' , (parents at: 1) nomParDefaut , ' passant par ' , (parents at: 2) nomParDefaut!

nomTypePrecis
	^'Perpendiculaire'! !

!IsardDroitePerpendiculaire class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#perpendiculaireA:passantPar: #(#IsardDroite #IsardPoint))! !

IsardMacro subclass: #IsardMacroTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMacroTest comment:
'macro de test. no comment.'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardMacroTest class
	instanceVariableNames: ''!


!IsardMacroTest class methodsFor: 'nomage'!

nomTypePrecis
	^'Test'! !

!IsardMacroTest class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#arg:arg:arg:  #(#IsardPointDepuisCoord #IsardPointDepuisCoord #IsardPointDepuisCoord  ))! !

!IsardMacroTest class methodsFor: 'instance creation'!

arg: Point2 arg: Point1 arg: Point3 
	| Droite1 Droite2 Droite3 Segment1 Segment2 Segment3 Droite4 Droite5 Droite6 Point4 Point5 Point6 Point7 Point8 Point9 Point10 Segment4 Segment5 Segment6 Point11 Point12 Point13 Droite7 Droite8 Droite9 Point14 Cercle1 Segment7 Point15 Cercle2 | 
	Droite1 := IsardDroite passantPar: Point1 etPar: Point2.
	Droite2 := IsardDroite passantPar: Point1 etPar: Point3.
	Droite3 := IsardDroite passantPar: Point2 etPar: Point3.
	Segment1 := IsardSegment joignant: Point1 avec: Point2.
	Segment2 := IsardSegment joignant: Point2 avec: Point3.
	Segment3 := IsardSegment joignant: Point3 avec: Point1.
	Droite4 := IsardDroite perpendiculaireA: Droite3 passantPar: Point1.
	Droite4 couleur: ColorValue blue.
	Droite5 := IsardDroite perpendiculaireA: Droite2 passantPar: Point2.
	Droite5 couleur: ColorValue blue.
	Droite6 := IsardDroite perpendiculaireA: Droite1 passantPar: Point3.
	Droite6 couleur: ColorValue blue.
	Point4 := IsardPoint milieuDe: Segment2.
	Point4 nom: 'A'''.
	Point5 := IsardPoint milieuDe: Segment3.
	Point5 nom: 'B'''.
	Point6 := IsardPoint milieuDe: Segment1.
	Point6 nom: 'C'''.
	Point7 := IsardPoint intersectionDeLaDroite: Droite4 avecLaDroite: Droite3.
	Point7 nom: 'A1'.
	Point8 := IsardPoint intersectionDeLaDroite: Droite5 avecLaDroite: Droite2.
	Point8 nom: 'B1'.
	Point9 := IsardPoint intersectionDeLaDroite: Droite6 avecLaDroite: Droite1.
	Point9 nom: 'C1'.
	Point10 := IsardPoint intersectionDeLaDroite: Droite4 avecLaDroite: Droite5.
	Point10 nom: 'H'.
	Segment4 := IsardSegment joignant: Point10 avec: Point1.
	Segment4 visibilite: false.
	Segment5 := IsardSegment joignant: Point10 avec: Point2.
	Segment5 visibilite: false.
	Segment6 := IsardSegment joignant: Point10 avec: Point3.
	Segment6 visibilite: false.
	Point11 := IsardPoint milieuDe: Segment4.
	Point11 nom: 'A2'.
	Point12 := IsardPoint milieuDe: Segment5.
	Point12 nom: 'B2'.
	Point13 := IsardPoint milieuDe: Segment6.
	Point13 nom: 'C2'.
	Droite7 := IsardDroite mediatriceDe: Segment1.
	Droite7 couleur: ColorValue red.
	Droite8 := IsardDroite mediatriceDe: Segment2.
	Droite8 couleur: ColorValue red.
	Droite9 := IsardDroite mediatriceDe: Segment3.
	Droite9 couleur: ColorValue red.
	Point14 := IsardPoint intersectionDeLaDroite: Droite7 avecLaDroite: Droite8.
	Point14 nom: 'O'.
	Cercle1 := IsardCercle deCentre: Point14 passantPar: Point1.
	Cercle1 couleur: ColorValue royalBlue.
	Segment7 := IsardSegment joignant: Point14 avec: Point10.
	Point15 := IsardPoint milieuDe: Segment7.
	Point15 nom: 'O'''.
	Cercle2 := IsardCercle deCentre: Point15 passantPar: Point7.
	Cercle2 couleur: ColorValue darkGreen ; epaisseur: 2.
	^nil! !

IsardCercle subclass: #IsardCercleDepuisCentrePoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardCercleDepuisCentrePoint comment:
'Cercle a partir d''un centre et d''un de ses points'!


!IsardCercleDepuisCentrePoint methodsFor: 'mise a jour'!

change
	| p |
	c := self parent1 p.
	p := self parent2 p.
	r := c dist: p! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardCercleDepuisCentrePoint class
	instanceVariableNames: ''!


!IsardCercleDepuisCentrePoint class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#deCentre:passantPar: #(#IsardPoint #IsardPoint))! !

!IsardCercleDepuisCentrePoint class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents
	^'Cercle de centre ' , (parents at: 1) nomParDefaut , ' passant par ' , (parents at: 2) nomParDefaut! !

IsardNombre subclass: #IsardAngleDeVecteurs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardAngleDeVecteurs comment:
'Angle entre deux IsardVecteur'!


!IsardAngleDeVecteurs methodsFor: 'mise a jour'!

change
	n := self parent2 v theta - self parent1 v theta / (2 * Float pi)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardAngleDeVecteurs class
	instanceVariableNames: ''!


!IsardAngleDeVecteurs class methodsFor: 'nomage'!

nomTypePrecis
	^'AngleDeVecteurs'! !

!IsardAngleDeVecteurs class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#angleEntre:et: #(#IsardVecteur #IsardVecteur))! !

!IsardAngleDeVecteurs class methodsFor: 'instance creation'!

angleEntre: v1 et: v2 
	^IsardAngleDeVecteurs newAvecParent: v1 etParent: v2! !

IsardObjet subclass: #IsardSegment
	instanceVariableNames: 'p1 p2 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardSegment comment:
'segment

Instance Variables:
	p1	<Point>	premiere extremit
	p2	<Point>	deuxieme extremit

'!


!IsardSegment methodsFor: 'accessing'!

estGeometrique
	"renvoie true si l'objet a une reelle consistance sur le papier"

	^true!

p1
	^p1!

p2
	^p2! !

!IsardSegment methodsFor: 'trace'!

retracePour: v sur: gc 
	gc displayLineFrom: (v trans: p1)
		to: (v trans: p2)! !

!IsardSegment methodsFor: 'distance-projection'!

projeteDuPoint: p 
	| u v t |
	v := p2 - p1.
	v isZero
		ifTrue: [^p1]
		ifFalse: 
			[u := p1 - p.
			t := (u dotProduct: v)
						/ (v dotProduct: v) negated.
			t < 0 ifTrue: [^p1].
			t > 1 ifTrue: [^p2].
			^t * v + p1]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardSegment class
	instanceVariableNames: ''!


!IsardSegment class methodsFor: 'instance creation'!

joignant: p1 avec: p2 
	^IsardSegmentDepuis2Points newAvecParent: p1 etParent: p2! !

!IsardSegment class methodsFor: 'nomage'!

nomType
	^'Segment'! !

IsardSegment subclass: #IsardSegmentDepuis2Points
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardSegmentDepuis2Points comment:
'segment a partir de deux points'!


!IsardSegmentDepuis2Points methodsFor: 'mise a jour'!

change
	p1 := self parent1 p.
	p2 := self parent2 p! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardSegmentDepuis2Points class
	instanceVariableNames: ''!


!IsardSegmentDepuis2Points class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#joignant:avec: #(#IsardPoint #IsardPoint))! !

!IsardSegmentDepuis2Points class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'[' , (parents at: 1) nomParDefaut , ',' , (parents at: 2) nomParDefaut , ']'! !

IsardObjet subclass: #IsardPoint
	instanceVariableNames: 'p '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPoint comment:
'point

Instance Variables:
	p	<Point>	coordonne du point
'!


!IsardPoint methodsFor: 'accessing'!

estGeometrique
	"renvoie true si l'objet a une reelle consistance sur le papier"

	^true!

p
	^p! !

!IsardPoint methodsFor: 'trace'!

retraceNomPour: v sur: gc 
	self nomParDefaut asComposedText displayOn: gc at: (v trans: p)!

retracePour: v sur: gc 
	| np |
	np := v trans: p.
	selectionnite
		ifTrue: 
			[gc displayLineFrom: np - 5 to: np + 5.
			gc displayLineFrom: np - (5 @ -5) to: np + (5 @ -5)]
		ifFalse: 
			[gc displayLineFrom: np - 1 to: np + 1.
			gc displayLineFrom: np - (1 @ -1) to: np + (1 @ -1)]! !

!IsardPoint methodsFor: 'distance-projection'!

projeteDuPoint: pt 
	^self p! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPoint class
	instanceVariableNames: ''!


!IsardPoint class methodsFor: 'instance creation'!

coord: c 
	^IsardPointDepuisCoord newAvecParent: c!

intersectionDeLaDroite: d1 avecLaDroite: d2 
	^IsardPointIntersection2Droites newAvecParent: d1 etParent: d2!

milieuDe: s
	^IsardPointMilieu newAvecParent: s!

surLeCercle: cercle controlePar: controle 
	^IsardPointSurCercle newAvecParent: cercle etParent: controle!

surLeLieu: lieu controlePar: controle 
	^IsardPointSurLieuPoint newAvecParent: lieu etParent: controle!

surLeSegment: segment controlePar: controle 
	^IsardPointSurSegment newAvecParent: segment etParent: controle!

transformationDe: point par: trans 
	^IsardPointApresTransformation newAvecParent: point etParent: trans! !

!IsardPoint class methodsFor: 'nomage'!

nomType
	^'Point'! !

IsardPoint subclass: #IsardPointApresTransformation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPointApresTransformation comment:
'point ayant subit une transformation'!


!IsardPointApresTransformation methodsFor: 'mise a jour'!

change
	p := self parent2 point: self parent1 p! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPointApresTransformation class
	instanceVariableNames: ''!


!IsardPointApresTransformation class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#transformationDe:par: #(#IsardPoint #IsardTransformation))! !

!IsardPointApresTransformation class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Point transform de ' , (parents at: 1) nomParDefaut , ' par ' , (parents at: 2) nomParDefaut! !

IsardPoint subclass: #IsardPointMilieu
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPointMilieu comment:
'point milieu d''un segment'!


!IsardPointMilieu methodsFor: 'mise a jour'!

change
	| s |
	s := self parent1.
	p := s p1 + s p2 / 2! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPointMilieu class
	instanceVariableNames: ''!


!IsardPointMilieu class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Milieu de ' , (parents at: 1) nomParDefaut! !

!IsardPointMilieu class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#milieuDe: #(#IsardSegment))! !

IsardPoint subclass: #IsardPointDepuisCoord
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPointDepuisCoord comment:
'point issu d''une coordonne'!


!IsardPointDepuisCoord methodsFor: 'mise a jour'!

change
	p := self parent1 p! !

!IsardPointDepuisCoord methodsFor: 'deplacement'!

estDeplacable
	^self parent1 calcule not!

vaEn: pt 
	self estDeplacable ifTrue: [self parent1 p: pt]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPointDepuisCoord class
	instanceVariableNames: ''!


!IsardPointDepuisCoord class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#coord: #(#IsardControleCoord))! !

IsardPoint subclass: #IsardPointIntersection2Droites
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPointIntersection2Droites comment:
'point intersection de deux droites'!


!IsardPointIntersection2Droites methodsFor: 'mise a jour'!

change
	| d1 d2 delta x y |
	d1 := self parent1.
	d2 := self parent2.
	delta := d1 a * d2 b - (d2 a * d1 b).
	delta isZero
		ifTrue: [self invalide]
		ifFalse: 
			[x := d1 b * d2 c - (d2 b * d1 c) / delta.
			y := d1 c * d2 a - (d2 c * d1 a) / delta.
			p := x @ y]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPointIntersection2Droites class
	instanceVariableNames: ''!


!IsardPointIntersection2Droites class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Intersection de ' , (parents at: 1) nomParDefaut , ' et de ' , (parents at: 2) nomParDefaut! !

!IsardPointIntersection2Droites class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#intersectionDeLaDroite:avecLaDroite: #(#IsardDroite #IsardDroite))! !

IsardNombre subclass: #IsardControleNombre
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardControleNombre comment:
'controleur de nombre'!


!IsardControleNombre methodsFor: 'calculs'!

calcule
	^false! !

!IsardControleNombre methodsFor: 'accessing'!

n: n0 
	n := n0.
	self aChange!

nSansChanger: n0 
	n := n0! !

!IsardControleNombre methodsFor: 'nomage'!

nomSpecifique
	"Implementation spcifique ncssitant l'instance"

	| s |
	s := WriteStream on: String new.
	s nextPutAll: self nomTypePrecis.
	s nextPutAll: ' controllant : '.
	filsDirects
		do: 
			[:f | 
			s nextPutAll: (f nom isNil ifFalse: [f nomParDefaut]
					ifTrue: [f nomTypePrecis]).
			s nextPutAll: (filsDirects last == f
					ifTrue: ['.']
					ifFalse: [','])].
	^s contents! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardControleNombre class
	instanceVariableNames: ''!


!IsardControleNombre class methodsFor: 'instance creation'!

n: n0 plan: p 
	^(self newAvecParent: p)
		n: n0! !

!IsardControleNombre class methodsFor: 'nomage'!

nomTypePrecis
	^'ControleNombre'! !

!IsardControleNombre class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#n:plan: #(#Object #IsardPlan))! !

!IsardControleNombre class methodsFor: 'parametres'!

parametres
	^#(#(#n #n:))! !

IsardDroite subclass: #IsardDroiteBissectrice
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardDroiteBissectrice comment:
'BIssectrice d''un angle defini par trois points'!


!IsardDroiteBissectrice methodsFor: 'mise a jour'!

change
	| p1 p2 p3 n |
	p1 := self parent1 p.
	p2 := self parent2 p.
	p3 := self parent3 p.
	p1 = p2 ifTrue: [^self invalide].
	p3 = p2 ifTrue: [^self invalide].
	n := (p1 - p2) normal + (p3 - p2) normal.
	a := n x.
	b := n y.
	c := (n dotProduct: p2) negated! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardDroiteBissectrice class
	instanceVariableNames: ''!


!IsardDroiteBissectrice class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#bissectriceDe:et:et: #(#IsardPoint #IsardPoint #IsardPoint))! !

!IsardDroiteBissectrice class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Bissectrice de [' , (parents at: 1) nomParDefaut , ',' , (parents at: 2) nomParDefaut , ',' , (parents at: 3) nomParDefaut , ']'! !

IsardObjet subclass: #IsardLieu
	instanceVariableNames: 'objets influants finesseMin finesseMax '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardLieu comment:
'lieu d''objets

Instance Variables:
	finesseMax	<Number>	finesse de decoupe maximale (cad: la plus fine)
	finesseMin	<Number>	finesse de decoupe minimale (cad: la plus grossiere)
	influants	<OrderedCollection>	collection des objets a mettre a jour pour calculer l''objet dont le lieu est dsir
	objets	<OrderedCollection>	collection des objets formant le lieu

'!


!IsardLieu methodsFor: 'initialisation'!

postInit
	"determine l'ensemble des objets a mettre a jour pour"
	"mettre a jour 'parent1' (l'objet du lieu)"
	"quand on modifie 'parent2' (le controlleur)"

	influants := self parent1 influantsSurSoiQuandOnChange: self parent2.
	super postInit! !

!IsardLieu methodsFor: 'mise a jour'!

metsAJourInfluants
	"mets a jour tous les objets necessaires a la construction"
	"de l'objet parametr du lieu"

	influants do: [:i | i miseAJour]! !

!IsardLieu methodsFor: 'accessing'!

estGeometrique
	"renvoie true si l'objet a une reelle consistance sur le papier"

	^true!

finesseMax
	^finesseMax isNil
		ifTrue: [self finesseMaxParDefaut]
		ifFalse: [finesseMax]!

finesseMax: f 
	finesseMax := f.
	self change.
	self aChange!

finesseMaxParDefaut
	^0.005!

finesseMin
	^finesseMin isNil
		ifTrue: [self finesseMinParDefaut]
		ifFalse: [finesseMin]!

finesseMin: f 
	finesseMin := f.
	self change.
	self aChange!

finesseMinParDefaut
	^0.1!

objets
	^objets! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardLieu class
	instanceVariableNames: ''!


!IsardLieu class methodsFor: 'instance creation'!

duPoint: p controlePar: controle 
	^IsardLieuPoint newAvecParent: p etParent: controle! !

!IsardLieu class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Lieu de ' , (parents at: 1) nomParDefaut , ' controll par ' , (parents at: 2) nomParDefaut!

nomType
	^'Lieu'! !

IsardVecteur subclass: #IsardVecteurDepuis2Points
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardVecteurDepuis2Points comment:
'vecteur a partir de deux points'!


!IsardVecteurDepuis2Points methodsFor: 'mise a jour'!

change
	p := self parent1 p.
	v := self parent2 p - p! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardVecteurDepuis2Points class
	instanceVariableNames: ''!


!IsardVecteurDepuis2Points class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#joignant:avec: #(#IsardPoint #IsardPoint))! !

IsardObjet subclass: #IsardTransformation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardTransformation comment:
'transformation gometrique

Subclasses must implement the following messages:
	transformation
		point:

'!


!IsardTransformation methodsFor: 'transformation'!

point: p 
	self subclassResponsibility! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardTransformation class
	instanceVariableNames: ''!


!IsardTransformation class methodsFor: 'nomage'!

nomType
	^'Transformation'! !

IsardTransformation subclass: #IsardSymetrieCentrale
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardSymetrieCentrale comment:
'transformation par une symetrie centrale'!


!IsardSymetrieCentrale methodsFor: 'transformation'!

point: p 
	| c |
	c := self centre.
	^c + c - p! !

!IsardSymetrieCentrale methodsFor: 'accessing'!

centre
	^self parent1 p! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardSymetrieCentrale class
	instanceVariableNames: ''!


!IsardSymetrieCentrale class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#deCentre: #(#IsardPoint))! !

!IsardSymetrieCentrale class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Symetrie de centre ' , (parents at: 1) nomParDefaut!

nomTypePrecis
	^'SymetrieCentrale'! !

!IsardSymetrieCentrale class methodsFor: 'instance creation'!

deCentre: c 
	^IsardSymetrieCentrale newAvecParent: c! !

IsardTransformation subclass: #IsardSymetrieAxiale
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardSymetrieAxiale comment:
'transformation par une symetrie axiale'!


!IsardSymetrieAxiale methodsFor: 'transformation'!

point: p 
	| proj |
	proj := self axe projeteDuPoint: p.
	^proj + proj - p! !

!IsardSymetrieAxiale methodsFor: 'accessing'!

axe
	^self parent1! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardSymetrieAxiale class
	instanceVariableNames: ''!


!IsardSymetrieAxiale class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#dAxe: #(#IsardDroite))! !

!IsardSymetrieAxiale class methodsFor: 'instance creation'!

dAxe: a 
	^IsardSymetrieAxiale newAvecParent: a! !

!IsardSymetrieAxiale class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Symetrie d''axe ' , (parents at: 1) nomParDefaut!

nomTypePrecis
	^'SymetrieAxiale'! !

IsardLieu subclass: #IsardLieuPoint
	instanceVariableNames: 'angleMax '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardLieuPoint comment:
'Lieu de points
on raffine jusqu''a finesseMax ou avoir un angle suffisament grand entre deux segments pour que cela ai l''air courbe.
Le calcul fait intervenir la mise a jour repetitive des influants sur le le point dont on calcule le lieu afin de creer une collection de points (le lieu).

Instance Variables:
	angleMax	<Number>	angle maximal entre deux segments (voir la definition de ''indicateurAngulaireDebut:milieu:fin:'')
'!


!IsardLieuPoint methodsFor: 'mise a jour'!

calculeEn: t 
	self parent2 nSansChanger: t.
	self metsAJourInfluants.
	^t -> (self parent1 validite
			ifTrue: [self parent1 p]
			ifFalse: [nil])!

change
	| on a1 a2 a3 |
	on := self parent2 n.
	objets := SortedCollection new.
	a1 := self calculeEn: 0.
	objets add: a1.
	a2 := self calculeEn: self finesseMin.
	objets add: a2.
	self finesseMin * 2
		to: 1
		by: self finesseMin
		do: 
			[:n | 
			a3 := self calculeEn: n.
			objets add: a3.
			a1 := self
						raffineDebut: a1
						milieu: a2
						fin: a3.
			a2 := a3].
	self parent2 nSansChanger: on.
	self metsAJourInfluants!

indicateurAngulaireDebut: p1 milieu: p2 fin: p3 
	| a b n |
	a := p2 - p1.
	b := p3 - p2.
	n := (a dotProduct: a)
				* (b dotProduct: b).
	^n isZero
		ifTrue: [0]
		ifFalse: [1 - ((a dotProduct: b) squared / n)]!

raffineDebut: a1 milieu: a2 fin: a3 
	| alpha na na2 |
	alpha := self
				indicateurAngulaireDebut: a1 value
				milieu: a2 value
				fin: a3 value.
	alpha < self angleMax | (a2 key - a1 key < self finesseMax) | (a3 key - a2 key < self finesseMax)
		ifTrue: [^a2]
		ifFalse: [a3 key - a2 key < (a2 key - a1 key)
				ifTrue: 
					[na := self calculeEn: a1 key + a2 key / 2.
					objets add: na.
					na := self
								raffineDebut: a1
								milieu: na
								fin: a2.
					^self
						raffineDebut: na
						milieu: a2
						fin: a3]
				ifFalse: 
					[na := self calculeEn: a2 key + a3 key / 2.
					objets add: na.
					na2 := self
								raffineDebut: a1
								milieu: a2
								fin: na.
					^self
						raffineDebut: na2
						milieu: na
						fin: a3]]! !

!IsardLieuPoint methodsFor: 'accessing'!

angleMax
	^angleMax isNil
		ifTrue: [self angleMaxParDefaut]
		ifFalse: [angleMax]!

angleMax: a 
	angleMax := a.
	self change.
	self aChange!

angleMaxParDefaut
	^0.03!

indexAvantLaCle: cle 
	| index low high |
	low := 1.
	high := objets size.
	
	[index := high + low // 2.
	low + 1 < high]
		whileTrue: [(objets at: index) key = cle
				ifTrue: [^index]
				ifFalse: [(objets at: index) key < cle
						ifTrue: [low := index]
						ifFalse: [high := index]]].
	^low! !

!IsardLieuPoint methodsFor: 'trace'!

retracePour: v sur: gc 
	gc displayPolyline: (objets collect: [:p | v trans: p value])! !

!IsardLieuPoint methodsFor: 'distance-projection'!

objetProjeteDuPoint: p 
	| curP curD newD |
	curP := objets first.
	curD := curP value dist: p.
	objets
		do: 
			[:newP | 
			newD := newP value dist: p.
			newD < curD
				ifTrue: 
					[curD := newD.
					curP := newP]].
	^curP!

projeteDuPoint: p 
	"tres approximatif pour l'instant"

	^(self objetProjeteDuPoint: p) value! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardLieuPoint class
	instanceVariableNames: ''!


!IsardLieuPoint class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#duPoint:controlePar: #(#IsardPoint #IsardControleNombre))! !

IsardTransformation subclass: #IsardRotation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardRotation comment:
'rotation'!


!IsardRotation methodsFor: 'accessing'!

angle
	^self parent2 n!

centre
	^self parent1 p! !

!IsardRotation methodsFor: 'transformation'!

point: p 
	| c a na |
	c := self centre.
	a := 2 * Float pi * self angle.
	na := a + (p - c) theta.
	^na cos @ na sin * (p - c) r + c! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardRotation class
	instanceVariableNames: ''!


!IsardRotation class methodsFor: 'instance creation'!

deCentre: c dAngle: nombre 
	^IsardRotation newAvecParent: c etParent: nombre! !

!IsardRotation class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Rotation de centre ' , (parents at: 1) nomParDefaut , ' d''angle ' , (parents at: 2) nomParDefaut!

nomTypePrecis
	^'Rotation'! !

!IsardRotation class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#deCentre:dAngle: #(#IsardPoint #IsardNombre))! !

IsardRotation subclass: #IsardMacroRotation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMacroRotation comment:
'macro creant une rotation a partir d''un point et d''un point sur un cercle'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardMacroRotation class
	instanceVariableNames: ''!


!IsardMacroRotation class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#deCentre:dAngleControleParPoint: #(#IsardPoint #IsardPointSurCercle))! !

!IsardMacroRotation class methodsFor: 'instance creation'!

deCentre: c dAngleControleParPoint: p 
	^IsardRotation newAvecParent: c etParent: p controle! !

IsardCercle subclass: #IsardCercleDepuisCentreRayon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardCercleDepuisCentreRayon comment:
'Cercle a partir d''un centre et de son rayon'!


!IsardCercleDepuisCentreRayon methodsFor: 'mise a jour'!

change
	c := self parent1 p.
	r := self parent2 n! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardCercleDepuisCentreRayon class
	instanceVariableNames: ''!


!IsardCercleDepuisCentreRayon class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#deCentre:etDeRayon: #(#IsardPoint #IsardNombre))! !

!IsardCercleDepuisCentreRayon class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Cercle de centre ' , (parents at: 1) nomParDefaut , ' et de rayon ' , (parents at: 2) nomParDefaut! !

IsardObjet subclass: #IsardCoord
	instanceVariableNames: 'p '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardCoord comment:
'Coordonne

Instance Variables:
	p	<Point>	valeur de la coordonne

'!


!IsardCoord methodsFor: 'accessing'!

p
	^p! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardCoord class
	instanceVariableNames: ''!


!IsardCoord class methodsFor: 'nomage'!

nomType
	^'Coordonne'! !

IsardCoord subclass: #IsardControleCoord
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardControleCoord comment:
'controleur de coordonne'!


!IsardControleCoord methodsFor: 'accessing'!

p: pt 
	p := pt.
	self aChange! !

!IsardControleCoord methodsFor: 'calculs'!

calcule
	^false! !

!IsardControleCoord methodsFor: 'nomage'!

nomSpecifique
	"Implementation spcifique ncssitant l'instance"

	| n |
	n := WriteStream on: String new.
	n nextPutAll: self nomTypePrecis.
	n nextPutAll: ' controllant : '.
	filsDirects
		do: 
			[:f | 
			n nextPutAll: (f nom isNil ifFalse: [f nomParDefaut]
					ifTrue: [f nomTypePrecis]).
			n nextPutAll: (filsDirects last == f
					ifTrue: ['.']
					ifFalse: [','])].
	^n contents! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardControleCoord class
	instanceVariableNames: ''!


!IsardControleCoord class methodsFor: 'instance creation'!

p: pt plan: plan 
	^(self newAvecParent: plan)
		p: pt! !

!IsardControleCoord class methodsFor: 'nomage'!

nomTypePrecis
	^'ControleCoordonne'! !

!IsardControleCoord class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#p:plan: #(#Object #IsardPlan))! !

!IsardControleCoord class methodsFor: 'parametres'!

parametres
	^#(#(#p #p:))! !

IsardLieuPoint subclass: #IsardMacroLieuPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMacroLieuPoint comment:
'macro creant un lieu de point a partir d''un point et d''un point sur un objet'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardMacroLieuPoint class
	instanceVariableNames: ''!


!IsardMacroLieuPoint class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#duPoint:controleParPoint: #(#IsardPoint #IsardPointSurObjet))! !

!IsardMacroLieuPoint class methodsFor: 'instance creation'!

duPoint: p controleParPoint: pointSurObjet 
	^IsardLieuPoint newAvecParent: p etParent: pointSurObjet controle! !

IsardDroite subclass: #IsardDroitePar2Points
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardDroitePar2Points comment:
'droite definie par deux points de celle-ci'!


!IsardDroitePar2Points methodsFor: 'mise a jour'!

change
	| p1 p2 |
	p1 := self parent1 p.
	p2 := self parent2 p.
	p1 = p2 ifTrue: [^self invalide].
	a := p2 y - p1 y.
	b := p1 x - p2 x.
	c := (a * p1 x + (b * p1 y)) negated! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardDroitePar2Points class
	instanceVariableNames: ''!


!IsardDroitePar2Points class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'(' , (parents at: 1) nomParDefaut , ',' , (parents at: 2) nomParDefaut , ')'! !

!IsardDroitePar2Points class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#passantPar:etPar: #(#IsardPoint #IsardPoint))! !

IsardPoint subclass: #IsardPointSurObjet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPointSurObjet comment:
'Point sur un objet

Subclasses must implement the following messages:
	deplacement
		vaEn:

'!


!IsardPointSurObjet methodsFor: 'accessing'!

contenant
	^self parent1!

controle
	^self parent2! !

!IsardPointSurObjet methodsFor: 'deplacement'!

estDeplacable
	^self controle calcule not!

fixeControle: n 
	self estDeplacable ifTrue: [self controle n: n]!

vaEn: pt 
	self subclassResponsibility! !

!IsardPointSurObjet methodsFor: 'trace'!

retraceNomPour: v sur: gc 
	| txt |
	txt := self nomParDefaut , '(' , self controle n printString , ')'.
	txt asComposedText displayOn: gc at: (v trans: p)! !

IsardPointSurObjet subclass: #IsardPointSurSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPointSurSegment comment:
'point sur un segment'!


!IsardPointSurSegment methodsFor: 'mise a jour'!

change
	| s l |
	s := self contenant.
	l := self controle n.
	p := l * s p1 + (1 - l * s p2)! !

!IsardPointSurSegment methodsFor: 'deplacement'!

vaEn: pt 
	| np s |
	s := self contenant.
	np := s projeteDuPoint: pt.
	self fixeControle: (s p2 - np) r / (s p1 - s p2) r! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPointSurSegment class
	instanceVariableNames: ''!


!IsardPointSurSegment class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#surLeSegment:controlePar: #(#IsardSegment #IsardControleNombre))! !

!IsardPointSurSegment class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Point sur le segment ' , (parents at: 1) nomParDefaut , ' control par ' , (parents at: 2) nomParDefaut! !

IsardPointSurObjet subclass: #IsardPointSurLieuPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPointSurLieuPoint comment:
'point sur un lieu de points'!


!IsardPointSurLieuPoint methodsFor: 'mise a jour'!

change
	| points n i p1 p2 lambda |
	points := self parent1 objets.
	n := self parent2 n.
	i := self parent1 indexAvantLaCle: n.
	p1 := points at: i.
	p2 := points at: i + 1.
	lambda := n - p1 key / (p2 key - p1 key).
	p := lambda * p2 value + (1 - lambda * p1 value)! !

!IsardPointSurLieuPoint methodsFor: 'deplacement'!

vaEn: pt 
	self fixeControle: (self contenant objetProjeteDuPoint: pt) key! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPointSurLieuPoint class
	instanceVariableNames: ''!


!IsardPointSurLieuPoint class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Point sur le lieu ' , (parents at: 1) nomParDefaut , ' control par ' , (parents at: 2) nomParDefaut! !

!IsardPointSurLieuPoint class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#surLeLieu:controlePar: #(#IsardLieuPoint #IsardControleNombre))! !

IsardPointSurLieuPoint subclass: #IsardMacroPointSurLieu
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMacroPointSurLieu comment:
'macro creant un point sur un lieu a partir d''un lieu'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardMacroPointSurLieu class
	instanceVariableNames: ''!


!IsardMacroPointSurLieu class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Point sur le lieu ' , (parents at: 1) nomParDefaut! !

!IsardMacroPointSurLieu class methodsFor: 'instance creation'!

surLeLieu: lieu 
	^IsardPointSurLieuPoint newAvecParent: lieu etParent: (IsardControleNombre n: 0 plan: lieu monPlan)! !

!IsardMacroPointSurLieu class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#surLeLieu: #(#IsardLieuPoint))! !

IsardPointSurObjet subclass: #IsardPointSurCercle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardPointSurCercle comment:
'point sur un cercle'!


!IsardPointSurCercle methodsFor: 'mise a jour'!

change
	| a ce |
	ce := self parent1.
	a := self parent2 n * Float pi * 2.
	p := a cos @ a sin * ce r + ce c! !

!IsardPointSurCercle methodsFor: 'deplacement'!

vaEn: pt 
	self fixeControle: (pt - self parent1 c) theta / (2 * Float pi)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardPointSurCercle class
	instanceVariableNames: ''!


!IsardPointSurCercle class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Point sur le cercle ' , (parents at: 1) nomParDefaut , ' control par ' , (parents at: 2) nomParDefaut! !

!IsardPointSurCercle class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#surLeCercle:controlePar: #(#IsardCercle #IsardControleNombre))! !

IsardPointSurCercle subclass: #IsardMacroPointSurCercle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMacroPointSurCercle comment:
'macro creant un point sur un cercle a partir d''un cercle'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardMacroPointSurCercle class
	instanceVariableNames: ''!


!IsardMacroPointSurCercle class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Point sur le cercle ' , (parents at: 1) nomParDefaut! !

!IsardMacroPointSurCercle class methodsFor: 'instance creation'!

surLeCercle: cercle 
	^IsardPointSurCercle newAvecParent: cercle etParent: (IsardControleNombre n: 0 plan: cercle monPlan)! !

!IsardMacroPointSurCercle class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#surLeCercle: #(#IsardCercle))! !

IsardPointSurSegment subclass: #IsardMacroPointSurSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Isard'!
IsardMacroPointSurSegment comment:
'macro creant un point sur un segment a partir d''un segment'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsardMacroPointSurSegment class
	instanceVariableNames: ''!


!IsardMacroPointSurSegment class methodsFor: 'methode de creation'!

methodeDeCreation
	^#(#surLeSegment: #(#IsardSegment))! !

!IsardMacroPointSurSegment class methodsFor: 'instance creation'!

surLeSegment: segment 
	^IsardPointSurSegment newAvecParent: segment etParent: (IsardControleNombre n: 0 plan: segment monPlan)! !

!IsardMacroPointSurSegment class methodsFor: 'nomage'!

nomSpecifiqueAvecParents: parents 
	^'Point sur le segment ' , (parents at: 1) nomParDefaut! !
IsardObjet initialize!


